/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus;

import java.io.IOException;
import java.util.List;

/**
 * <p>
 *  <code>Binder</code> provides an interface for all constraint
 *    representations to use. It defines the contract for conversion
 *    between an arbitrary constraint representation (XML Schema, DTD,
 *    Relax schema, etc) to a set of Zeus 
 *    <code>{@link Binding}</code>s.
 * </p><p>
 *   This interface also provides uniformity in the overall handling
 *     of constraints; in other words, dealing with an XML Schema is
 *     generally fundamentally different than dealing with a DTD.
 *     However, working with a <code>DTDBinder</code> is identical,
 *     from an external perspective, to working with a
 *     <code>SchemaBinder</code>.
 * </p>
 *
 * @author Brett McLaughlin
 * @author Maciej Zawadzki
 * @version 1.0
 */
public interface Binder {
    
    /**
     * XXX: One thing I'm annoyed about here: I can't specify
     *      that the constructor should take as a single argument
     *      a <code>Source</code>. I guess that's OK, as some input
     *      types may be something else, but it's still a bit 
     *      annoying. Perhaps I need to flesh out
     *      <code>BaseBinder</code> or something?
     */

    /**
     * <p>
     *  This method allows the developer to set a prefix that 
     *    will be prepended to the name of every generated 
     *    interface and class.  
     * </p>
     *
     * @param String prefix that will be prepended to the 
     *        name of every generated interface and class
     */
    public void setNamePrefix(String namePrefix);
    
    /**
     * <p>
     *  This method allows the developer to get the prefix that 
     *    will be prepended to the name of every generated
     *    interface and class.  Returns an empty string in the 
     *    case that a prefix is not prepended.
     * </p>
     *
     * @return <code>String</code> prefix that will be prepended 
     *         to the name of every generated interface and class
     */
    public String getNamePrefix();

    /**
     * <p>
     *  This method allows the developer to specify that all 
     *    simple XML elements (elements with 
     *    <code>#PCDATA</code> content) should map to data 
     *    members on parent member classes rather than their 
     *    own classes.
     * </p>
     *
     * @param collapseSimpleElements true if simple elements 
     *        should be collapsed
     */
    public void setCollapseSimpleElements(
        boolean collapseSimpleElements);
    
    /**
     * <p>
     *   This method allows the developer to determine whether 
     *    all simple XML elements (elements with 
     *    <code>#PCDATA</code> content) will be mapped to 
     *    data members on parent member classes rather 
     *    than their own classes.
     * </p>
     *
     * @return <code>boolean</code> true if simple elements 
     *         will be collapsed
     */
    public boolean isCollapsingSimpleElements();

    /**
     * <p>
     *  This is integral portion of the <code>Binder</code>. It
     *    is responsible for returning a Zeus representation of
     *    the set of constraints that this binding represents,
     *    resulting from the input representation (which could
     *    be an XML Schema, DTD, Relax schema, etc.).
     * </p>
     *
     * @return <code>List</code> - the resultant
     *         <code>Binding</code>s from conversion of
     *         constraints.
     * @throws <code>IOException</code> when errors in reading
     *         input occur.     
     */
    public List getBindings() throws IOException;

}