/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus;

/**
 * <p>
 *  <code>Binding</code> represents an arbitrary Java construct,
 *    and provides a representation-independent means of showing
 *    how a constraint (presumably from an XML Schema, DTD, Relax
 *    schema, etc.) maps to Java. It provides a layer of representation
 *    between the constraint representation and generated Java.
 * </p><p>
 *  It also allows for uniformity in mapping. For example, the DTD
 *    <code>ELEMENT</code> and XML Schema <code>element</code>
 *    constructs, while different, should both be mapped to the
 *    same implementation of a <code>Binding</code> in Zeus. In this
 *    way, constraints that represent semantically equivalent
 *    constructs result in identical sets of bindings.
 * </p>
 *
 * @author Brett McLaughlin
 * @version 1.0
 */
public interface Binding {

    /**
     * <p>
     *  This will return the Java name of the binding. This should
     *    be the valid Java name of the binding, which is most
     *    often a variable name.  This name will be the result of
     *    any conversions that have to be done if the XML name of
     *    the binding is not a valid Java identifier.
     * </p>
     * <p>
     *  For example, for a <code>List</code> that is associated 
     *    with an instance variable named "myList", invoking
     *    <code>getJavaName()</code> would return "myList." For
     *    a custom object type "Foo", associated with an instance
     *    variable "foo", invoking <code>getJavaName()</code>
     *    would return "foo".
     * </p>
     * <p>
     *  It's also important to note that there are cases when
     *    this would return <code>null</code>. For an object that
     *    is top-level, and is not a member variable of any other
     *    object, no name will be associated with the binding.
     * </p>
     *
     * @return <code>String</code> - the Java name of the binding.
     */
    public String getJavaName();

    /**
     * <p>
     *  This will return the XML name of the binding. This should
     *    be the actual XML name of the binding, which is most
     *    often a variable name.
     * </p>
     * <p>
     *  For example, for a <code>List</code> that is associated 
     *    with an instance variable named "myList", invoking
     *    <code>getXMLName()</code> would return "myList." For a
     *    custom object type "Foo", associated with an instance
     *    variable "foo", invoking <code>getXMLName()</code>
     *    would return "foo".
     * </p>
     * <p>
     *  It's also important to note that there are cases when
     *    this would return <code>null</code>. For an object that
     *    is top-level, and is not a member variable of any other
     *    object, no name will be associated with the binding.
     * </p>
     *
     * @return <code>String</code> - the XML name of the binding.
     */
    public String getXMLName();
    
    /**
     * <p>
     *  This will set the XML name of the binding. For more details
     *    on the name of a binding, see 
     *    <code>{@link #getXMLName()}</code>.
     * </p>
     *
     * @param xmlName <code>String</code> XML name of the binding.
     */
    public void setXMLName(String xmlName);
    
    /**
     * <p>
     *  This will return the Java type associated with the
     *    binding. This will always be the fully qualified
     *    name of the Java object, excepting cases where
     *    the object is in default included packages.
     * </p>
     * <p>
     *  For example, an object of type <code>org.bby.Foo</code>
     *    would return "org.bby.Foo" when this method is invoked.
     *    However, an object of type <code>java.lang.String</code>
     *    would only return "String." In all cases, the type
     *    returned here is qualified enough to not require
     *    any import statements in generated code.
     * </p>
     *
     * @return <code>String</code> - the Java type of the 
     *         binding.
     */
    public String getJavaType();
    
    /**
     * <p>
     *  This will set the Java type for this binding. For
     *    more details on Java types and bindings, see
     *    <code>{@link #getJavaType()}</code>.
     * </p>
     *
     * <b>Note:</b> Should this do an internal check in
     *    implementations to ensure full qualification, like
     *    with Class.forName() or something? If so, notes
     *    should be made here to that effect.
     *
     * @param javaType <code>String</code> the Java type
     *        for this binding.
     */
    public void setJavaType(String javaType);
}

