/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus;

import java.io.File;
import java.io.IOException;

/**
 * <p>
 *  <code>Generator</code> is the portion of Zeus that will convert
 *    a set of constraints, visible through a <code>{@link Binding}</code>,
 *    and output them as generated Java classes (to a 
 *    <code>{@link Result}</code>.
 * </p>
 *
 * @author Brett McLaughlin
 * @author Maciej Zawadzki
 * @version 1.0
 */
public interface Generator {
    
    /** The default <code>Collection</code> class */
    public static final String DEFAULT_COLLECTION_CLASS = "java.util.List";

    /**
     * XXX: Since this is the user interface, there is almost
     *      certainly a set of exceptions that need to exist
     *      and be thrown from the <code>generate()</code> method.
     *      I'll have to add those as they come up.
     */
    
    /**
     * <p>
     *  This will set the <code>Collection</code> class to use anytime that
     *    a collection of values need to be stored. The default value for
     *    this class is stored in <code>{@link #DEFAULT_COLLECTION_CLASS}</code>.
     *    The class specified should be an implementation of the
     *    <code>java.util.Collection</code> interface.
     * </p>
     *
     * @param collectionClass <code>String</code> name of collection to use.
     */
    public void setCollectionClass(String collectionClass);
    
    /**
     * <p>
     *  This will return the current class used for <code>Collection</code>
     *    properties.
     * </p>
     *
     * @return <code>String</code> - name of class used for <code>Collection</code>s.
     */
    public String getCollectionClass();

    /**
     * <p>
     *  Sets the Java package to generate new interfaces and implementation
     *    within. For example, supplying the value <tt>com.foo.bar</tt> 
     *    to this method and then generating classes, the classes 
     *    would all be in the <tt>com.foo.bar</tt> package after generation.
     * </p>
     * 
     * @param javaPackage <code>String</code> the Java package to 
     *        generate interfaces and implementation classes within.
     */
    public void setJavaPackage(String interfacePackage);

    /**
     * <p>
     *  Returns the Java package to generate interfaces within.
     * </p>
     * 
     * @return <code>String</code> - the Java package to unmarshall 
     *         interfaces to.
     */
    public String getInterfacePackage();

    /**
     * <p>
     *  Sets the Java package to generate new interfaces within. For example,
     *    supplying the value <tt>com.foo.bar</tt> to this method and then
     *    generating classes, the interfaces would all be in the 
     *    <tt>com.foo.bar</tt> package after generation.
     * </p>
     * 
     * @param interfacePackage <code>String</code> the Java package to 
     *        generate interfaces within.
     */
    public void setInterfacePackage(String interfacePackage);

    /**
     * <p>
     *  Returns the Java package to generate implementation classes
     *    within.
     * </p>
     * 
     * @return <code>String</code> - the Java package to generate 
     *         implementation classes within.
     */
    public String getImplementationPackage();

    /**
     * <p>
     *  Sets the Java package to generate new interfaces within. For example,
     *    supplying the value <tt>com.foo.bar</tt> to this method and then
     *    generating classes, the interfafces would all be in the 
     *    <tt>com.foo.bar</tt> package after generation.
     * </p>
     * 
     * @param implPackage <code>String</code> the Java package to 
     *        generate interfaces within.
     */
    public void setImplementationPackage(String implPackage);

    /**
     * <p>
     *  This sets the output directory to output generated classes to.
     * </p>
     *
     * @param outputDir the name of the base directory where 
     *        all generated classes should be placed
     */
    public void setOutputDirectory(String outputDir);
    
    /**
     * <p>
     *  This sets the output directory to output generated classes to.
     * </p>
     *
     * @param outputDir the <code>File</code> (directory) where 
     *        all generated classes should be placed
     */
    public void setOutputDirectory(File outputDir);

    /**
     * <p>
     *  This will return <code>true</code> if the generator is set to
     *    generate <code>Serializable</code> classes, or else
     *    <code>false</code>.
     * </p>
     *
     * @return <code>boolean</code> - whether classes will be serializable.
     */
    public boolean generateAsSerializable();

    /**
     * <p>
     *  This will allow for setting the generator to generate serializable
     *    classes. By default, classes are <i>not</i> set to be 
     *    serializable.
     * </p>
     *
     * @param generateAsSerializable whether or not to generate serializable
     *        classes
     */
    public void setGenerateAsSerializable(boolean generateAsSerializable);

    /**
     * <p>
     *  This allows a binding to be ignored. This is an intermediary step towards
     *    allowing binding schemas. The name of the binding should be passed
     *    in here, and no property generation for it will occur.
     * </p>
     *
     * @param bindingName the name of the binding to ignore.
     */
    public void setIgnoreBinding(String bindingName);

    /**
     * <p>
     *  This will convert from a set of constraints to Java classes.
     *    It accesses those constraints through the
     *    <code>{@link Binding}</code> interface, which provides
     *    them in a representation-independent format, and then
     *    converts them into Java code.
     * </p>
     *
     * @param binder <code>Binder</code> with ability to convert from
     *        constraints to Zeus <code>Binding</code>s.
     * @throws <code>IOException</code> - when errors in writing to the
     *         supplied <code>Result</code> occur.     
     */
    public void generate(Binding binding) throws IOException;
}
