/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus;

import org.enhydra.zeus.result.StreamResult;
import org.enhydra.zeus.util.CapitalizationUtils;
import org.enhydra.zeus.util.NamingUtils;

import java.io.IOException;
import java.io.OutputStream;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

// JDOM imports
import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.DocType;
import org.jdom.Element;
import org.jdom.output.XMLOutputter;

/**
 * <p>
 *  <code>Marshaller</code> takes a Java <code>Object</code> instance and writes
 *    out an XML representation of that object, with each property (variable)
 *    of the instance and its value.
 * </p>
 *
 * @author Brett McLaughlin
 */
public class Marshaller {

    /** Constant used to signel whether debug messages should be printed out */
    static public boolean DEBUG = false;
    
    /** List of methods to ignore in marshalling. */
    private List methodsToIgnore;

    /**
     * Prefix to be prepended to the name of every generated 
     *   interface and class.
     */
    protected String namePrefix = "";
    
    /**
     * <p>
     *  Simple constructor.
     * </p>
     */
    public Marshaller() {
        methodsToIgnore = new LinkedList();
        methodsToIgnore.add("getClass");
        methodsToIgnore.add("getValue");
    }

    /**
     * <p>This will set an (additional) method to ignore in marshalling. The
     *   argument to this method should be the method name to ignore, without
     *   arguments or parenthesis. To ignore the method "getFactory()", the
     *   value would simply be "getFactory" for this method.</p>
     *
     * @param methodToIgnore name of method to ignore.
     */
    public void ignoreMethod(String methodToIgnore) {
        methodsToIgnore.add(methodToIgnore);
    }
   
    /**
     * <p>
     *  This method allows the developer to set a prefix that 
     *    will be prepended to the name of every generated 
     *    interface and class.  
     * </p>
     *
     * @param String prefix that will be prepended to the 
     *        name of every generated interface and class
     */
    public void setNamePrefix(String namePrefix) {
        if (namePrefix != null) {
            this.namePrefix = namePrefix;
        }
    }

    /**
     * <p>
     *  This method allows the developer to get the prefix that 
     *    will be prepended to the name of every generated
     *    interface and class.  Returns an empty string in the 
     *    case that a prefix is not prepended.
     * </p>
     *
     * @return <code>String</code> prefix that will be prepended 
     *         to the name of every generated interface and class
     */
    public String getNamePrefix() {
        return namePrefix;
    }
    
    /**
     * <p>
     *  This method is the public entry point for marshalling an object into
     *    an XML instance document. 
     * </p>
     *
     * @param obj <code>Object</code> to convert to XML.
     * @param result <code>Result</code> to write XML to.
     * @throws <code>IOException</code> when errors in output occur.
     */
    public void marshal(Object obj, Result result) 
        throws IOException {

        if (obj instanceof UnmarshalledObject) {
            marshal((UnmarshalledObject)obj, result);
        } else {
            marshal(new UnmarshalledObject(obj), result);
        }
    }

    /**
     * <p>
     *  This method is the public entry point for marshalling an object into
     *    an XML instance document. It requires that an object previously
     *    unmarshalled by Zeus (an instance of <code>UnmarshalledObject</code>)
     *    be supplied.
     * </p>
     *
     * @param obj <code>UnmarshalledObject</code> to convert to XML.
     * @param result <code>Result</code> to write XML to.
     * @throws <code>IOException</code> when errors in output occur.
     */
    public void marshal(UnmarshalledObject obj, Result result) 
        throws IOException {

        // Root Element is the start of recursion
        Element root = getXMLRepresentation(obj.getObject());
        Document doc = new Document(root);

        // Set a doctype if needed
        String systemID = obj.getSystemID();
        String publicID = obj.getPublicID();
        if ((systemID != null) || (publicID != null)) {
            DocType docType = new DocType(root.getName(), publicID, systemID);
            doc.setDocType(docType);
        }
        
        // XXX: For now, I don't know how we would write to anthing but
        //   a StreamResult. So this is a temp hack, for sure!
        if (result instanceof StreamResult) {
            StreamResult streamResult = (StreamResult)result;

            // Use 2 space indentation and line feeds
            XMLOutputter outputter = new XMLOutputter("  ", true);            
            outputter.output(doc, streamResult.getWriter());
            streamResult.getWriter().flush();
        } else {
            throw new IllegalArgumentException(
                "Marshaller currently only works with StreamResults.");
        }
    }

    /**
     * <p>
     *  This is the granular portion of binding; a Java <code>Object</code> is
     *    converted into an XML element (in JDOM form), using recursion for any
     *    children.
     * </p>
     * 
     * @param obj <code>Object</code> to get the XML element representation for.
     * @return <code>Element</code> - representation of 
     *         <code>Object</code> in XML.
     * @throws <code>IOException</code> when errors occur in binding.
     */
    private Element getXMLRepresentation(Object obj) throws IOException {
        Class objectClass = obj.getClass();

        // Get the name of the element for this object
        String objectName = objectClass.getName();

        // Strip the package name.
        int index = -1;
        if ((index = objectName.lastIndexOf('.')) != -1) {
            objectName = objectName.substring((index + 1));
        }

        // Get the attributes order, if possible
        boolean zeusGenerated = false;
        String[] attributes = new String[0];
        String[] childElements = new String[0];
        try {

            // Get the element's XML name
            Field xmlName = 
                objectClass.getField(ZeusDefaults.XML_NAME_FIELD);
            objectName = (String) xmlName.get(obj);

            // Get attributes
            Field attributesField = 
                objectClass.getField(ZeusDefaults.ATTRIBUTE_ARRAY_FIELD);
            attributes = (String[])attributesField.get(obj);
            
            // Get elements
            Field elementsField = 
                objectClass.getField(ZeusDefaults.ELEMENT_ARRAY_FIELD);
            childElements = (String[])elementsField.get(obj);

            zeusGenerated = true;
        } catch (NoSuchFieldException e) {
            // Not a Zeus-generated object
        } catch (IllegalAccessException e) {
            throw new IOException(e.getMessage());
        } catch (SecurityException e) {
            throw new IOException(e.getMessage());
        }

        // Strip the name prefix
        if (objectName.startsWith(namePrefix)) {
            objectName = objectName.substring(namePrefix.length());
        }

        // If this is an "Impl" class, remove that from the name
        if ((index = objectName.indexOf("Impl")) != -1) {
            objectName = objectName.substring(0, index);
        }

        Element element = new Element(objectName);
        if (DEBUG) {
            System.out.println("Marshaller.getXMLRepresentation: "+
                                objectName);
        }

        // Deal with the object; either by direct access or introspection
        if (zeusGenerated) {
            // Deal with attributes
            for (int i=0; i<attributes.length; i++) {
                String attribute = attributes[i];

                // Build the method name
                String methodName = new StringBuffer("get")
                    .append(CapitalizationUtils.initialUpper(attribute))
                    .toString();

                // Lookup the method
                try {
                    Method method = objectClass.getMethod(methodName, null);
                    String value = (String)method.invoke(obj, null);

                    if (value == null) {
                        continue;
                    } else {
                        element.addAttribute(attribute, value);
                    }
                } catch (NoSuchMethodException e) {
                    throw new IOException("An expected method, " +
                        methodName + ", was not found, and is required.");
                } catch (IllegalAccessException e) {
                    throw new IOException(e.getMessage());
                } catch (InvocationTargetException e) {
                    throw new IOException(e.getMessage());
                } catch (SecurityException e) {
                    throw new IOException(e.getMessage());
                }
            }

            // Deal with elements
            for (int i=0; i<childElements.length; i++) {
                String childElementName = childElements[i];
                if (DEBUG) {
                    System.out.println("Marshaller.getXMLRepresentation: "+
                                       objectName+"."+childElementName);
                }
 
                // Build the "get" method name
                String methodName = new StringBuffer("get")
                    .append(childElementName)
                    .toString();

                // Lookup the "get" method
                try {
                    Method method = objectClass.getMethod(methodName, null);
                    Object value = method.invoke(obj, null);

                    if (value == null) {
                        continue;
                    } else {
                        // check if simple
                        String valueClassName = value.getClass().getName();
                        Element childElement = null;
                        if (valueClassName.startsWith("java.lang")) {
                            // simple
                            childElement = new Element(childElementName);
                            childElement.setText(String.valueOf(value));
                        } else {
                            // not simple
                            childElement = getXMLRepresentation(value);
                        }
                        element.addContent(childElement);
   
                        // We're done; bail out
                        continue;
                    }
                } catch (NoSuchMethodException e) {
                    // Not in the form of a "get" method
                } catch (IllegalAccessException e) {
                    throw new IOException(e.getMessage());
                } catch (InvocationTargetException e) {
                    throw new IOException(e.getMessage());
                } catch (SecurityException e) {
                    throw new IOException(e.getMessage());
                }                

                // If we got here, we're still looking for the method to call
                // Build the "List" method name
                methodName = new StringBuffer("get")
                    .append(childElementName)
                    .append("List")
                    .toString();

                // Lookup the "List" method
                try {
                    Method method = objectClass.getMethod(methodName, null);
                    List values = (List)method.invoke(obj, null);
                    Object value = null;
                    for (Iterator j = values.iterator(); j.hasNext(); ) {
                        value = j.next();
                        if (value == null) {
                            continue;
                        } else {
                            // check if simple
                            String valueClassName = value.getClass().getName();
                            Element childElement = null;
                            if (valueClassName.startsWith("java.lang")) {
                                // simple
                                childElement = new Element(childElementName);
                                childElement.setText(String.valueOf(value));
                            } else {
                                // not simple
                                childElement = getXMLRepresentation(value);
                            }
                            element.addContent(childElement);

                            // We're done; bail out
                            continue;
                        }
                    }

                    // We're done, so bail out
                    continue;
                } catch (NoSuchMethodException e) {
                    // Not in the form of a "List" method
                } catch (IllegalAccessException e) {
                    throw new IOException(e.getMessage());
                } catch (InvocationTargetException e) {
                    throw new IOException(e.getMessage());
                } catch (SecurityException e) {
                    throw new IOException(e.getMessage());
                }                 
            }


            // Even if the element does not have child elements, it still can 
            // have content.  Check for that text content here.
            String methodName = "getValue";

                // Lookup the "Value" method
                try {
                    Method method = objectClass.getMethod(methodName, null);
                    String value = (String)method.invoke(obj, null);
                    element.setText(value);
                } catch (NoSuchMethodException e) {
                //throw new IOException("The getValue() method " +
                //    "could not be found.");
                } catch (IllegalAccessException e) {
                    throw new IOException(e.getMessage());
                } catch (InvocationTargetException e) {
                    throw new IOException(e.getMessage());
                } catch (SecurityException e) {
                    throw new IOException(e.getMessage());
                }
           
        // Deal with object through reflection
        } else {
            Method[] methods = objectClass.getMethods();
            for (int i=0; i<methods.length; i++) {
                // Deal with attributes
                Method method = methods[i];
                String methodName = method.getName();
                if ((methodName.startsWith("get")) && 
                    (!methodsToIgnore.contains(methodName)) &&
                    (!methodName.endsWith("List"))) {

                    // Get the value for this method
                    try {
                        Object o = method.invoke(obj, new Object[] { });
                        // Ensure that the element exists
                        if (o == null) {
                            continue;
                        }

                        // Remove the "get" and lower the initial letter
                        String propertyName =
                            NamingUtils.getXMLElementNameFromAccessor(
                                method.getName());
                    
                        // Determine if it's primitive 
                        if (o.getClass().getName().startsWith("java.lang.")) {
                            // If it's a primitive, add as an attribute
                            element.addAttribute(propertyName, o.toString());
                        } else {
                            // ... otherwise, recurse and add new element as child
                            element.addContent(getXMLRepresentation(o));
                        }
                    } catch (IllegalAccessException e) {
                        throw new IOException(e.getMessage());
                    } catch (InvocationTargetException e) {
                        throw new IOException(e.getMessage());
                    }
                } else if ((methodName.startsWith("get")) &&
                           (methodName.endsWith("List"))) {
                    // Deal with List data
                    try {
                        List items = (List)method.invoke(obj, new Object[] { });
                        for (Iterator j = items.iterator(); j.hasNext(); ) {
                            element.addContent(getXMLRepresentation(j.next()));
                        }
                    } catch (IllegalAccessException e) {
                        throw new IOException(e.getMessage());
                    } catch (InvocationTargetException e) {
                        throw new IOException(e.getMessage());
                    }                
                } else if (methodName.equals("getValue")) {
                    // Deal with Data field
                    try {
                        String content = 
                            (String)method.invoke(obj, new Object[] { });
                        element.setText(content);
                    } catch (IllegalAccessException e) {
                        throw new IOException(e.getMessage());
                    } catch (InvocationTargetException e) {
                        throw new IOException(e.getMessage());
                    }                    
                }
            }
        }

        return element;
    }
}

