/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus;

import java.io.IOException;

/**
 * <p>
 *  <code>Result</code> provides an interface for all output
 *    means. It details the required contract that other
 *    portions of the Zeus XML data binding framework must
 *    use for processing to an arbitrary output.
 * </p>
 * <p>
 *  All output facilities should implement this interface, and
 *    be named <code>XXXResult</code> where <i>XXX</i> reflects the
 *    type of output handled. For example, output to I/O
 *    streams is handled by 
 *    <code>{@link org.enhydra.zeus.result.StreamResult}</code>.
 * </p>
 *
 * @author Brett McLaughlin
 * @version 1.0
 */
public interface Result {

    /**
     * <p>
     *  This will return the system ID associated with
     *    this <code>Result</code>. This is generally in
     *    the form of a URI.
     * </p>
     *
     * @return <code>String</code> - the system ID for the
     *         <code>Result</code>.
     */
    public String getSystemID();
    
    /**
     * <p>
     *  This will set the system ID for this 
     *    <code>Result</code>. This is important to use,
     *    even when output is in the form of an output stream
     *    (see 
     *    <code>{@link org.enhydra.zeus.result.StreamResult}</code>)
     *    for resolving external references, such as to a DTD.
     * </p>
     *
     * @param systemID <code>String</code> system ID to use.
     */    
    public void setSystemID(String systemID);
    
    /**
     * <p>
     *  This will write a character stream to the output
     *    facility associated with this <code>Result</code>.
     *    No additional formatting is added, so line feeds,
     *    tabs, or other special characters should be handled
     *    by the code performing output.
     * </p>
     *
     * @param output <code>String</code> to output.
     * @throws <code>IOException</code> - when errors in output occur.     
     */
    public void write(String output) throws IOException;
}