/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus;

import java.io.IOException;

// JDOM imports
import org.jdom.Document;

// SAX imports
import org.xml.sax.EntityResolver;

/**
 * <p>
 *  <code>Source</code> provides an interface for all input
 *    means. It details the required contract that other
 *    portions of the Zeus XML data binding framework must
 *    use for processing of an arbitrary input.
 * </p>
 * <p>
 *  All input sources should implement this interface, and
 *    be named <code>XXXSource</code> where <i>XXX</i> reflects the
 *    type of input handled. For example, input from I/O
 *    streams is handled by 
 *    <code>{@link org.enhydra.zeus.source.StreamSource}</code>.
 * </p>
 *
 * @author Brett McLaughlin
 * @version 1.0
 */
public interface Source {

    /**
     * <p>
     *  This will return the system ID associated with
     *    this <code>Source</code>. This is generally in
     *    the form of a URI.
     * </p>
     *
     * @return <code>String</code> - the system ID for the
     *         <code>Source</code>.
     */
    public String getSystemID();
    
    /**
     * <p>
     *  This will set the system ID for this 
     *    <code>Source</code>. This is important to use,
     *    even when input is in the form of an input stream
     *    (see 
     *    <code>{@link org.enhydra.zeus.source.StreamSource}</code>)
     *    for resolving external references, such as to a DTD.
     * </p>
     *
     * @param systemID <code>String</code> system ID to use.
     */
    public void setSystemID(String systemID);
    
    /**
     * <p>
     *  This will return a JDOM <code>Document</code> that
     *    represents the input source.
     * </p>
     *
     * @return <code>Document</code> - the JDOM representation
     *         of the input source.
     * @throws <code>IOException</code> - when construction of
     *         a <code>Document</code> generates errors.     
     */
    public Document getDocument() throws IOException;
    /**
     * <p>
     *  This will set an EntityResolver for this 
     *    <code>Source</code>. This is passed to the underlying
     *    parser to use.
     * </p>
     *
     * @param entityResolver <code>org.xml.sax.EntityResolver</code> Entity 
     * Resolver to use.
     */
    public void setEntityResolver(EntityResolver entityResolver); 
}
