/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.binder;

import java.io.IOException;
import java.util.List;

// Zeus imports
import org.enhydra.zeus.Binder;
import org.enhydra.zeus.Binding;
import org.enhydra.zeus.Source;

/**
 * <p>
 *  <code>{@link Binder}</code> provides an interface for all constraint
 *    representations to use. It defines the contract for conversion
 *    between an arbitrary constraint representation (XML Schema, DTD,
 *    Relax schema, etc) to a set of Zeus 
 *    <code>{@link Binding}</code>s.
 * </p><p>
 *  This implementation of <code>Binder</code> handles the
 *    basic functionality of dealing with input so that
 *    other implementations don't have to code these methods.
 *    Thus, <code>XXXBinder</code> classes should extend
 *    this class, rather than directly implementing
 *    <code>Binder</code>, and will get this functionality
 *    "for free." 
 * </p>
 *
 * @author Brett McLaughlin
 * @author Maciej Zawadzki
 * @version 1.0
 */
public abstract class BaseBinder implements Binder {

    /** The <code>{@link Source}</code> to read input from */
    protected Source source;

    /**
     * Prefix to be prepended to the name of every generated 
     *   interface and class.
     */
    protected String namePrefix = "";
    
    /**
     * Determines whether simple elements map to data members 
     *   on parent clases or to their own classes.
     */
    protected boolean collapseSimpleElements = false;

    /**
     * <p>
     *  This constructor takes in a <code>{@link Source}</code>
     *    to read input from and allow generation of the 
     *    <code>{@link Binding}</code>s from it.
     * </p>
     *
     * @param source <code>Source</code> to read input from.
     */
    public BaseBinder(Source source) {
        this.source = source;
    }

    /**
     * <p>
     *  This method allows the developer to set a prefix that 
     *    will be prepended to the name of every generated 
     *    interface and class.  
     * </p>
     *
     * @param String prefix that will be prepended to the 
     *        name of every generated interface and class
     */
    public void setNamePrefix(String namePrefix) {
        if (namePrefix != null) {
            this.namePrefix = namePrefix;
        }
    }
    
    /**
     * <p>
     *  This method allows the developer to get the prefix that 
     *    will be prepended to the name of every generated
     *    interface and class.  Returns an empty string in the 
     *    case that a prefix is not prepended.
     * </p>
     *
     * @return <code>String</code> prefix that will be prepended 
     *         to the name of every generated interface and class
     */
    public String getNamePrefix() {
        return namePrefix;
    }

    /**
     * <p>
     *  This method allows the developer to specify that all 
     *    simple XML elements (elements with 
     *    <code>#PCDATA</code> content) should map to data 
     *    members on parent member classes rather than their 
     *    own classes.
     * </p>
     *
     * @param collapseSimpleElements true if simple elements 
     *        should be collapsed
     */
    public void setCollapseSimpleElements(
        boolean collapseSimpleElements) {

        this.collapseSimpleElements = collapseSimpleElements;
    }
    
    /**
     * <p>
     *   This method allows the developer to determine whether 
     *    all simple XML elements (elements with 
     *    <code>#PCDATA</code> content) will be mapped to 
     *    data members on parent member classes rather 
     *    than their own classes.
     * </p>
     *
     * @return <code>boolean</code> true if simple elements 
     *         will be collapsed
     */
    public boolean isCollapsingSimpleElements() {
        return collapseSimpleElements;
    }

    /**
     * <p>
     *  This is integral portion of the <code>Binder</code>. It
     *    is responsible for returning a Zeus representation of
     *    the set of constraints that this binding represents,
     *    resulting from the input representation (which could
     *    be an XML Schema, DTD, Relax schema, etc.).
     * </p><p>
     *  There is a temptation to implement a lazy-loading mechanism
     *    here, but that should <i>not</i> be done. That's because
     *    the <code>{@link Source}</code> may stay the same but the actual
     *    input (abstracted by the <code>Source</code> may change
     *    "under the covers." So don't implement lazy-loading here!
     * </p>
     *
     * @return <code>List</code> - the resultant
     *         <code>Binding</code>s from conversion of
     *         constraints.
     * @throws <code>IOException</code> when errors in reading
     *         input occur.     
     */
    public abstract List getBindings() throws IOException;

}