/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.binder;

import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;
import java.util.BitSet;

// Zeus imports
import org.enhydra.zeus.Binder;
import org.enhydra.zeus.Binding;
import org.enhydra.zeus.binding.Property;
import org.enhydra.zeus.binding.AtomicProperty;
import org.enhydra.zeus.binding.Container;
import org.enhydra.zeus.binding.ContainerProperty;
import org.enhydra.zeus.source.DTDSource;
import org.enhydra.zeus.util.NamingUtils;
import org.enhydra.zeus.util.DTDUtils;

// DTDParser imports
import com.wutka.dtd.DTD;
import com.wutka.dtd.DTDAny;
import com.wutka.dtd.DTDAttribute;
import com.wutka.dtd.DTDCardinal;
import com.wutka.dtd.DTDContainer;
import com.wutka.dtd.DTDDecl;
import com.wutka.dtd.DTDElement;
import com.wutka.dtd.DTDEmpty;
import com.wutka.dtd.DTDEnumeration;
import com.wutka.dtd.DTDItem;
import com.wutka.dtd.DTDMixed;
import com.wutka.dtd.DTDName;
import com.wutka.dtd.DTDPCData;

/**
 * <p>
 *  <code>DTDBinder</code> implements the <code>{@link Binder}</code>
 *    interface and allows generation of Zeus
 *    <code>{@link Binding}</code>s from a DTD.
 * </p>
 *
 * @author Brett McLaughlin
 * @author Steve Witten
 * @author Maciej Zawadzki
 * @version 1.0
 */
public class DTDBinder extends BaseBinder {

    /** The DTD being processed */
    private DTD dtd = null;

    /**
     * <p>
     *  This constructor takes in a <code>{@link DTDSource}</code>
     *    to read an XML Schema from and allow generation of the 
     *    <code>{@link Binding}</code>s from it.
     * </p>
     *
     * @param source <code>Source</code> to read input from.
     */
    public DTDBinder(DTDSource source) {
        super(source);
    }

    /**
     * <p>
     *  This is integral portion of the <code>Binder</code>. It
     *    is responsible for returning a Zeus representation of
     *    the set of constraints that this binding represents,
     *    resulting from the supplied DTD.
     * </p>
     *
     * @return <code>List</code> - the resultant
     *         <code>Binding</code>s from conversion of
     *         constraints.
     * @throws <code>IOException</code> when errors in reading
     *         input occur.     
     */
    public List getBindings() throws IOException {
        // Get the DTD
        dtd = ((DTDSource)source).getDTD();
        
        // Create a list for the bindings
        List bindings = new LinkedList();
    
        Iterator i = dtd.getItemsByType(DTDElement.class).iterator();
        while (i.hasNext()) {
            DTDElement dtdElement = (DTDElement)i.next();

            if ((DTDUtils.isSimpleElement(dtdElement)) && 
                (isCollapsingSimpleElements())) {
                // skip this element if it is simple and 
                //   we are collapsing simple elements
                continue;
            } else {
                bindings.add(convertToBinding(dtdElement));
            } 
        }
    
        return bindings;
    }
    
    /**
     * <p>
     *  This will take a single DTD element (in the form of a
     *    <code>DTDElement</code>) and convert it to a Zeus
     *    <code>Binding</code> object.
     * </p>
     *
     * @param dtdElement <code>DTDElement</code> to convert to
     *        Zeus <code>Binding</code>.
     * @return <code>Binding</code> - the converted Zeus <code>Binding</code>.
     */
    private Binding convertToBinding(DTDElement dtdElement) {
        ContainerProperty container = 
            new ContainerProperty(namePrefix + 
                dtdElement.getName());
            
        // Deal with the content of the DTDElement
        handleContent(container, dtdElement.getContent());
        
        // Deal with the attributes of the DTDElement
        handleAttributes(container, dtdElement);
    
        return container;
    }
    
    /**
     * <p>
     *  This will take an element from a DTD (in <code>DTDElement</code>
     *    form) and create Zeus <code>{@link AtomicProperty}</code> objects
     *    from its attributes, adding them to the supplied
     *    <code>{@link Container}</code>.
     * </p>
     *
     * @param container <code>Container</code> to add properties to.
     * @param dtdElement <code>DTDElement</code> to read attributes of.
     */
    private void handleAttributes(Container container, DTDElement dtdElement) {
        Hashtable elementAttributes = dtdElement.attributes;
        Enumeration e = elementAttributes.elements();
        while (e.hasMoreElements()) {
            DTDAttribute attribute = (DTDAttribute)e.nextElement();
            /**
             * Create the attribute; all DTD attributes are 
             *   simply Java <code>String</code>s, since
             *   there is no typing in DTDs.
             */
            AtomicProperty atomicProperty;
            DTDDecl decl = attribute.getDecl();
            
            BitSet t = new BitSet();
            t.set(Property.ACCESS_PRIVATE);
            t.set(Property.SOURCE_ATTLIST);
            
            // should the decl be stored for future reference?
            //   maybe in the code generator?
            // we need to handle #REQUIRED, #IMPLIED, and #VALUE too...
            
            if (decl != null && decl.equals(DTDDecl.FIXED)) {
               t.set(Property.MUTABILITY_FINAL);
               t.set(Property.STORAGE_STATIC);
            }    
            atomicProperty = 
                   new AtomicProperty(
                       attribute.getName(),
                       
                    // should this be based on attribute.getType()?
                    //    |
                    // vvvvvvvv
                       "String", t,
                       (attribute.getType() instanceof DTDEnumeration) ? 
                           ((DTDEnumeration)attribute.getType()).getItemsVec() :
                           null,
                           attribute.getDefaultValue());
            container.addProperty(atomicProperty);
        }        
    }

    /**
     * <p>
     *  This will take an element's content (in <code>DTDItem</code>
     *    form) and create Zeus <code>{@link AtomicProperty}</code> objects
     *    from its attributes, adding them to the supplied
     *    <code>{@link Container}</code>.
     * </p>
     *
     * @param conainer <code>Container</code> to add properties to.
     * @param content <code>DTDItem</code> with element's content.
     */
    private void handleContent(Container container, DTDItem content) {
        // Cycle through the various types of content
        if (content instanceof DTDEmpty) {       
            // We're done - this is an empty element
        } else if (content instanceof DTDPCData) {
            // Add a single String property
            String propertyName = "value";
            AtomicProperty property = 
                new AtomicProperty(propertyName, "String");
            container.addProperty(property);
        } else if (content instanceof DTDName) { 
            // Create the correct type of property
            String propertyName = 
                ((DTDName)content).getValue();
            DTDCardinal cardinality = ((DTDName)content).getCardinal();
            
            // Get the type
            String itemName = ((DTDName)content).getValue();
            String propertyType = "String";
            if (!isSimpleElement(itemName) || !isCollapsingSimpleElements()) {
                // if it is not simple or we are not collapsing simple elements
                propertyType = namePrefix +
                    NamingUtils.getJavaClassName(propertyName);
            }
                
            // Generate a single property
            BitSet t = new BitSet();
            t.set(Property.SOURCE_ELEMENT);
            AtomicProperty property = 
                new AtomicProperty(propertyName, propertyType, t);            

            // Handle cardinality - either a list or a single property
            if ((cardinality.equals(DTDCardinal.ZEROMANY)) ||
                (cardinality.equals(DTDCardinal.ONEMANY))) {
                    
                property.setIsCollection(true);
            }
            
            container.addProperty(property);
        } else if (content instanceof DTDAny) {
            // XXX: Not sure WHAT to do here? Maybe a List?
            String propertyName = "anyData";
            AtomicProperty property =
                new AtomicProperty(propertyName, "Object");
            property.setIsCollection(true);
            container.addProperty(property);
        } else if (content instanceof DTDContainer) {       
            // Get each item and handle it
            Vector items = ((DTDContainer)content).getItemsVec();

            // Deal with container cardinality
            DTDCardinal cardinality = ((DTDContainer)content).cardinal;
            boolean hasCardinality = false;
            if ((cardinality.equals(DTDCardinal.ZEROMANY)) ||
                (cardinality.equals(DTDCardinal.ONEMANY))) {
                hasCardinality = true;
            }

            for (Iterator i = items.iterator(); i.hasNext(); ) {
                DTDItem subContent = (DTDItem)i.next();

                // Deal with transitive nature of cardinality on containers
                handleContent(container, subContent);
            }
        }    
    }

    /**
     * <p>
     *  Helper method looks up the element with the specified name in 
     *    the DTD and determines whether the element is simple.  
     * </p>
     * <p>
     *  An element is simple if its content is PCDATA.  In that case, 
     *    the element does not have to be represented by a class but 
     *    can be mapped to a field within the parent element's class.
     * </p>
     * 
     * @param name the name of the element to lookup
     * @return <code>boolean</code> - whether the element is "simple"
     */
    private boolean isSimpleElement(String name) {
        boolean result = true;
        
        DTDElement temp = (DTDElement)dtd.elements.get(name);
        if (temp != null) {
            result = DTDUtils.isSimpleElement(temp);
        } 
        return result;
    }
    
}