/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.binding;

// Zeus imports
import java.util.BitSet;
import java.util.Vector;

/**
 * <p>
 *  <code>AtomicProperty</code> implements the 
 *    <code>{@link org.enhydra.zeus.Binding}</code> and
 *    <code>{@link Property}</code> interfaces, and provides
 *    for a simple name/value property representation.
 * </p><p>
 *  An <code>AtomicProperty</code> will most commonly represent
 *    simple typed values in Java, such as a <code>String</code>,
 *    <code>int</code>, or <code>Date</code>.
 * </p>
 *
 * @author Brett McLaughlin
 * @author Steve Witten
 * @author Maciej Zawadzki
 * @version 1.0
 */
public class AtomicProperty extends BaseProperty {

    /**
     * <p>
     *  This will create a new <code>AtomicProperty</code> with
     *    the specified information.
     * </p>
     *
     * @param xmlName the <code>String</code> XML name of this property.
     * @param javaType the Java class name of the property type.
     * @param accessModifier the <code>int<code> constant for the
     *        property's access modifier.
     * @see <code>{@link Property#ACCESS_PRIVATE}</code>
     * @see <code>{@link Property#ACCESS_PROTECTED}</code>
     * @see <code>{@link Property#ACCESS_PUBLIC}</code>          
     * @see <code>{@link Property#STORAGE_STATIC}</code>
     * @see <code>{@link Property#MUTABILITY_VOLATILE}</code>
     * @see <code>{@link Property#MUTABILITY_FINAL}</code>          
     */
    public AtomicProperty(String xmlName, String javaType, 
                          BitSet modifier, Vector enumeration, Object defaultValue) {
        super();
        this.xmlName = xmlName;
        this.javaType = javaType;
        this.modifier = modifier;
        this.defaultValue = defaultValue;
        this.enumeration = enumeration;
    }
    
    /**
     * <p>
     *  This will create a new <code>AtomicProperty</code> with
     *    the specified information. The property will be, by default,
     *    <code>{@link Property#ACCESS_PRIVATE}</code>.
     * </p>
     *
     * @param xmlName the <code>String</code> XML name of this property.
     * @param javaType the Java class name of the property type.         
     */
    public AtomicProperty(String xmlName, String javaType) {
        super();
        
        BitSet modifier = new BitSet();
        modifier.set(Property.ACCESS_PRIVATE);

        this.xmlName = xmlName;
        this.javaType = javaType;
        this.modifier = modifier;
        this.defaultValue = null;
        this.enumeration = null;
    }

    /**
     * <p>
     *  This will create a new <code>AtomicProperty</code> with
     *    the specified information. The property will be
     *    <code>{@link Property#ACCESS_PRIVATE}</code>.
     * </p>
     *
     * @param xmlName the <code>String</code> XML name of this property.
     * @param javaType the Java class name of the property type.        
     * @param modifier the BitSet containing modifiers for this property 
     */
    public AtomicProperty(String xmlName, String javaType, BitSet modifier) {
        super();
        modifier.set(Property.ACCESS_PRIVATE);

        this.xmlName = xmlName;
        this.javaType = javaType;
        this.modifier = modifier;
        this.defaultValue = null;
        this.enumeration = null;
    }
}

