/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.binding;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * <p>
 *  <code>{@link Container}</code> implements the 
 *    <code>{@link org.enhydra.zeus.Binding}</code>
 *    interface and defines behavior for a binding that can contain other
 *    bindings (usually <code>{@link Property}</code> implementations). It
 *    is used to represent objects which have nested objects.
 * </p><p>
 *  This base implementation of <code>Container</code> defines the common
 *    functionality for all <code>Container</code> implementations, removing
 *    a need for them to duplicate code for this functionality. All 
 *    implementations of <code>Container</code> should extend this class
 *    rather than directly implementing the <code>Container</code> interface.
 * </p>
 *
 * @author Brett McLaughlin
 * @version 1.0
 */
public abstract class BaseContainer extends BaseBinding implements Container {

    /** 
     * The <code>{@link Property}</code> objects this
     *  <code>Container</code> holds.
     */  
    protected List properties;
    
    /**
     * <p>
     *  This will add a <code>{@link Property}</code> to
     *    the member variables of this <code>Container</code>.
     * </p><p>
     *  It is important to note that this is simply an instance
     *    variable being added to (in most cases) a custom
     *    Java class definition. That variable/property
     *    doesn't have a value (although it might have a
     *    default value) until marshalling and unmarshalling
     *    occurs.
     * </p>
     *
     * @param property <code>Property</code> to add.
     */
    public void addProperty(Property property) {
        if (properties == null) {
            properties = new LinkedList();
        }
        
        properties.add(property);
    }
    
    /**
     * <p>
     *  This will remove a <code>{@link Property}</code> 
     *    from this <code>Container</code>, given the
     *    property's name. If a successful removal occurs,
     *    the boolean value <code>true</code> is retruned.
     *    If no <code>Property<code> is found with the supplied
     *    name, the boolean value <code>false</code> is returned.
     * </p>
     *
     * @param propertyName <code>String</code> name of 
     *        <code>Property</code> to remove.
     * @return <code>boolean</code> - indicates whether
     *         the specified named <code>Property</code>
     *         was found and removed.
     */
    public boolean removeProperty(String propertyName) {
        if (properties == null) {
            return false;
        }
        
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            if (property.getJavaName().equals(propertyName)) {
                i.remove();
                return true;
            }
        }
        
        return false;
    }    
    
    /**
     * <p>
     *  This will return a list of all the 
     *    <code>{@link Property}</code> objects that
     *    this <code>Container</code> has. If there are
     *    none, this will return an empty <code>List</code>.
     * </p>
     *
     * @return <code>List</code> - properties for this
     *         <code>Container</code>.
     */
    public List getProperties() {
        if (properties == null) {
            return new LinkedList();
        }
        
        return properties;
    }    
    
    /**
     * <p>
     *  This will clear all the properties for this <code>Container</code>.
     * </p>
     */
    public void clearProperties() {
        if (properties != null) {
            properties.clear();
        }
    }    

}

