/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.binding;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.BitSet;
import java.util.Vector;

/**
 * <p>
 *  <code>ContainerProperty</code> implements the 
 *    <code>{@link org.enhydra.zeus.Binding}</code>,
 *    <code>{@link Property}</code>, and
 *    <code>{@link Container}</code> interfaces, and provides
 *    for a complex object that is itself a property.
 * </p><p>
 *  A <code>ContainerProperty</code> will most commonly represent
 *    a Java <code>Object</code> that has properties of its own 
 *    (like any other <code>Container</code), but that also is a
 *    property itself, of some other Java <code>Object</code>.
 * </p>
 *
 * @author Brett McLaughlin
 * @version 1.0
 */
public class ContainerProperty extends BaseBinding implements Property, Container {

    /**
     * Note that since Java doesn't allow multiple inheritance, this class
     *   can't simple extend <code>{@link BaseProperty}</code> and
     *   <code>{@link BaseContainer}</code>, which is really what it should
     *   do. Instead, all those methods must be duplicated here, which is really
     *   sort of crummy.
     */

    /** The access level modifier for this <code>Property</code> */
    protected BitSet modifier;
    
    /** Whether this <code>Property</code> represents a <code>Collection</code> */
    protected boolean isCollection;    
    
    /** The default value for this <code>Property</code> */
    protected Object defaultValue;    
    
    /** 
     * The <code>{@link Property}</code> objects this
     *  <code>Container</code> holds.
     */  
    protected List properties;    
    
    /**
     * <p>
     *  This will create a new <code>ContainerProperty</code> with
     *    the specified information.
     * </p>
     *
     * @param name the <code>String</code> name of this property.
     * @param modifier the <code>BitSet<code> representing the
     *        property's modifier(s).
     * @see <code>{@link Property#ACCESS_PRIVATE}</code>
     * @see <code>{@link Property#ACCESS_PROTECTED}</code>
     * @see <code>{@link Property#ACCESS_PUBLIC}</code>          
     */    
    public ContainerProperty(String xmlName, BitSet modifier, Object defaultValue) {
        this.isCollection = false;
        this.xmlName = xmlName;
        this.modifier = modifier;
        this.defaultValue = defaultValue;
    }
    
    /**
     * <p>
     *  This will create a new <code>ContainerProperty</code> with
     *    the specified information. The property will be, by default,
     *    <code>{@link Property#ACCESS_PRIVATE}</code>.
     * </p>
     *
     * @param name the <code>String</code> name of this property.        
     */    
    public ContainerProperty(String xmlName) {
        BitSet modifier = new BitSet();
        modifier.set(Property.ACCESS_PRIVATE);
        this.isCollection = false;
        this.xmlName = xmlName;
        this.modifier = modifier;
        this.defaultValue = null;
    }

    /**
     * <p>
     *  This will set the modifiers for a property. The
     *    value submitted must be in the form of an <code>int</code>,
     *    which should correspond to one of the constants defined
     *    (<code>{@link #ACCESS_PRIVATE}</code>,
     *     <code>{@link #ACCESS_PROTECTED}</code>,
     *     <code>{@link #ACCESS_PUBLIC}</code>,
     *     <code>{@link #STORAGE_STATIC}</code>,
     *     <code>{@link #MUTABILITY_VOLATILE}</code>,
     *     or <code>{@link #MUTABILITY_FINAL}</code>. By default,
     *    all properties will be <code>private</code>
     *    (<code>ACCESS_PRIVATE</code>).
     * </p>
     *
     * @param modifier <code>BitSet</code>representing modifiers.
     * @see #ACCESS_PRIVATE
     * @see #ACCESS_PROTECTED
     * @see #ACCESS_PUBLIC          
     * @see #STORAGE_STATIC
     * @see #MUTABILITY_VOLATILE
     * @see #MUTABILITY_FINAL
     */
    public void setModifier(BitSet modifier) {
        this.modifier = modifier;
    }
    
    /**
     * <p>
     *  This will return the access level modifier for a property. The
     *    value returned will be in the form of an <code>int</code>,
     *    which will correspond to one of the constants defined
     *    (<code>{@link #ACCESS_PRIVATE}</code>,
     *     <code>{@link #ACCESS_PROTECTED}</code>,
     *     <code>{@link #ACCESS_PUBLIC}</code>,
     *     <code>{@link #STORAGE_STATIC}</code>,
     *     or <code>{@link #MUTABILITY_FINAL}</code>.
     * </p>
     *
     * @return <code>BitSet</code> - modifiers.
     * @see #ACCESS_PRIVATE
     * @see #ACCESS_PROTECTED
     * @see #ACCESS_PUBLIC          
     * @see #STORAGE_STATIC
     * @see #MUTABILITY_VOLATILE
     * @see #MUTABILITY_FINAL
     */    
    public BitSet getModifier() {
        return modifier;    
    }
    
    /**
     * <p>
     *  This will return the Java <code>String</code> representation
     *    of this <code>Property</code>'s modifiers. For
     *    example, <code>{@link #ACCESS_PRIVATE}</code> would be
     *    converted to "private".
     * </p>
     *
     * @return <code>String</code> - Java representation
     * @see #ACCESS_PRIVATE
     * @see #ACCESS_PROTECTED
     * @see #ACCESS_PUBLIC   
     * @see #STORAGE_STATIC
     * @see #MUTABILITY_VOLATILE
     * @see #MUTABILITY_FINAL            
     */
    public String getModifierString() {
        String t = new String();
        if (modifier.get(ACCESS_PRIVATE))
            t += "private";
        else if (modifier.get(ACCESS_PROTECTED))
            t += "protected";
        else if (modifier.get(ACCESS_PUBLIC))
            t += "public";
        else
            throw new UnsupportedOperationException(
                "Modifier must set access modifiers.");
        
        if (modifier.get(STORAGE_STATIC))
           t += " static";
           
        if (modifier.get(MUTABILITY_VOLATILE))
           t += " volatile";
        else if (modifier.get(MUTABILITY_FINAL))
                t += " final";
                
        return t;
    }    
    
    /**
     * <p>
     *  This will whether or not this <code>Property</code>
     *    is a <code>Collection</code> (in other words, the property
     *    represents a collection of values). By default, properties
     *    are all singular values.
     * </p>
     *
     * @param isCollection <code>true</code> is multiple values can be stored,
     *        or else <code>false</code>.
     */
    public void setIsCollection(boolean isCollection) {
        this.isCollection = isCollection;
    }
    
    /**
     * <p>
     *  This will indicate whether this <code>Property</code> represents
     *    a <code>Collection</code> of values (resulting in a <code>true</code>
     *    result from this method), or a singular value (resulting in a
     *    <code>false</code> result).
     * </p>
     *
     * @return <code>boolean</code> - whether or not this <code>Property</code>
     *         represents a <code>Collection<code>.
     */
    public boolean isCollection() { 
        return isCollection;
    }    
    
    /**
     * <p>
     *  This indicates whether or not this <code>Property</code> has a
     *    a default value set.
     * </p>
     *
     * @return <code>boolean</code> - whether there is a default value set.
     */ 
    public boolean hasDefaultValue() {
        return defaultValue == null;
    }
    
    /**
     * <p>
     *  This will set the default value of the property. Since no typing
     *    is available at this point, a simple Java <code>Object</code>
     *    is allowed as the type supplied. As a result, any errors in 
     *    mismatches between object type and allowed paramater type will
     *    occur at runtime, when class generation takes place. Supplying
     *    a value here essentially results in:
     * <code><pre>
     *    public class Foo {
     *
     *        private String myString = "some default value";
     *
     *        public String getMyString() {
     *            return myString;
     *        }
     *
     *        public void setMyString(String myString) {
     *            this.myString = myString;
     *        }
     * 
     *        // Other methods and properties
     *    }
     * </pre></code>
     * </p><p>
     *  Also, note that data binding users who supply their own class
     *    implementations will <b>LOSE THIS DEFAULT VALUE</b>, as the interface
     *    alone cannot specify a default value. So use this carefully!
     * </p>
     *
     * @param defaultValue <code>Object</code> to be used as default value.
     */
    public void setDefaultValue(Object defaultValue) {
        this.defaultValue = defaultValue;    
    }
    
    /**
     * <p>
     *  This will retrieve the default value associated with this property,
     *    or <code>null</code> if there is not one. For more information on
     *    default property values, see 
     *    <code>{@link #setDefaultValue(Object)}</code>.
     * </p>
     *
     * @return <code>Object</code> - default value of the property.
     */
    public Object getDefaultValue() {
        return defaultValue;
    }
    
    /**
     * <p>
     *  This will indicate if this <code>Property</code> has a set of
     *    allowed values (an enumeration) specified for it. For
     *    <code>ContainerProperty</code> implementations, this will
     *    <i>always</i> be false.
     * </p>
     *
     * @return <code>boolean</code> - whether an enumeration is specified.
     */
    public boolean hasEnumeration() {
        return false;
    }

    /**
     * <p>
     *  This will set a list (enumeration) of allowed values for this
     *    <code>Property</code>. This complies with the <code>Property</code>
     *    interface, but does nothing for containers.
     * </p>
     *
     * @param enumeration the <code>Vector</code> of allowed values.
     */
    public void setEnumeration(Vector enumeration) {
       return;
    }
   
    /**
     * <p>
     *  This returns the <code>Vector</code> of allowed values for this
     *    <code>Property</code>, or <code>null</code> if there is none.
     *    For a <code>ContainerProperty</code>, this will <i>always</i>
     *    return <code>null</code>.
     * </p>
     *
     * @return <code>Vector</code> - the allowed values.
     */
    public Vector getEnumeration() {
       return null;
    }
          
    /**
     * <p>
     *  This will add a <code>{@link Property}</code> to
     *    the member variables of this <code>Container</code>.
     * </p><p>
     *  It is important to note that this is simply an instance
     *    variable being added to (in most cases) a custom
     *    Java class definition. That variable/property
     *    doesn't have a value (although it might have a
     *    default value) until marshalling and unmarshalling
     *    occurs.
     * </p>
     *
     * @param property <code>Property</code> to add.
     */
    public void addProperty(Property property) {
        if (properties == null) {
            properties = new LinkedList();
        }
        
        properties.add(property);
    }
    
    /**
     * <p>
     *  This will remove a <code>{@link Property}</code> 
     *    from this <code>Container</code>, given the
     *    property's name. If a successful removal occurs,
     *    the boolean value <code>true</code> is retruned.
     *    If no <code>Property<code> is found with the supplied
     *    name, the boolean value <code>false</code> is returned.
     * </p>
     *
     * @param propertyName <code>String</code> name of 
     *        <code>Property</code> to remove.
     * @return <code>boolean</code> - indicates whether
     *         the specified named <code>Property</code>
     *         was found and removed.
     */
    public boolean removeProperty(String propertyName) {
        if (properties == null) {
            return false;
        }
        
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            if (property.getJavaName().equals(propertyName)) {
                i.remove();
                return true;
            }
        }
        
        return false;
    }    
    
    /**
     * <p>
     *  This will return a list of all the 
     *    <code>{@link Property}</code> objects that
     *    this <code>Container</code> has. If there are
     *    none, this will return an empty <code>List</code>.
     * </p>
     *
     * @return <code>List</code> - properties for this
     *         <code>Container</code>.
     */
    public List getProperties() {
        if (properties == null) {
            return new LinkedList();
        }
        
        return properties;
    }    
    
    /**
     * <p>
     *  This will clear all the properties for this <code>Container</code>.
     * </p>
     */
    public void clearProperties() {
        if (properties != null) {
            properties.clear();
        }
    }    
}