/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.binding;

import java.util.BitSet;
import java.util.Vector;

// Zeus imports
import org.enhydra.zeus.Binding;

/**
 * <p>
 *  <code>Property</code> implements the <code>{@link Binding}</code>
 *    interface and defines behavior for a property of a Java class.
 *    These properties have modifiers, types, and names, as well
 *    as a value.
 * </p>
 *
 * @author Brett McLaughlin
 * @author Steve Witten
 * @author Maciej Zawadzki
 * @version 1.0
 */
public interface Property extends Binding {

    /** Access Modifier: <code>private</code> */
    public static final int ACCESS_PRIVATE      = 1;

    /** Access Modifier: <code>private</code> */
    public static final int ACCESS_PROTECTED    = 2;
    
    /** Access Modifier: <code>public</code> */        
    public static final int ACCESS_PUBLIC       = 3;

    /** Storage Modifier: <code>static</code> */            
    public static final int STORAGE_STATIC      = 4;
    
    /** Mutability Modifier: <code>volatile</code> */
    public static final int MUTABILITY_VOLATILE = 5;
    
    /** Mutability Modifier: <code>final</code> */    
    public static final int MUTABILITY_FINAL    = 6;
    
    /** Mutability Modifier: <code>synchronized</code> */
    public static final int MUTABILITY_SYNCHRONIZED = 7;
    
    /** Mutability Modifier: <code>transient</code> */
    public static final int MUTABILITY_TRANSIENT = 8;

    /** The source of the property; in this case, an attribute */
    public static final int SOURCE_ATTLIST = 9;

    /** The source of the property; in this case, an element */
    public static final int SOURCE_ELEMENT = 10;

    /**
     * <p>
     *  This will set the modifier for a property. The
     *    value submitted must be in the form of a <code>BitSet</code>,
     *    in which the set bits correspond to the constants defined
     *    (<code>{@link #ACCESS_PRIVATE}</code>,
     *     <code>{@link #ACCESS_PROTECTED}</code>,
     *     <code>{@link #ACCESS_PUBLIC}</code>,
     *     <code>{@link #STORAGE_STATIC}</code>,
     *     <code>{@link #MUTABILITY_VOLATILE}</code>,
     *     <code>{@link #MUTABILITY_FINAL}</code> or
     *     <code>{@link #MUTABILITY_SYNCHRONIZED}</code>). By default,
     *    all properties will be <code>private</code>
     *    (<code>ACCESS_PRIVATE</code>).
     * </p>
     *
     * @param modifier <code>int</code> constant for access level.
     * @see #ACCESS_PRIVATE
     * @see #ACCESS_PROTECTED
     * @see #ACCESS_PUBLIC
     * @see #STORAGE_STATIC
     * @see #MUTABILITY_VOLATILE
     * @see #MUTABILITY_FINAL
     * @see #MUTABILITY_SYNCHRONIZED                    
     */
    public void setModifier(BitSet modifier);
    
    /**
     * <p>
     *  This will return the modifier for a property.The
     *    value returned will be in the form of a <code>BitSet</code>,
     *    in which the set bits correspond to the constants defined
     *    (<code>{@link #ACCESS_PRIVATE}</code>,
     *     <code>{@link #ACCESS_PROTECTED}</code>,
     *     <code>{@link #ACCESS_PUBLIC}</code>,
     *     <code>{@link #STORAGE_STATIC}</code>,
     *     <code>{@link #MUTABILITY_VOLATILE}</code>,
     *     <code>{@link #MUTABILITY_FINAL}</code> or
     *     <code>{@link #MUTABILITY_SYNCHRONIZED}</code>). By default,
     *    all properties will be <code>private</code>
     *    (<code>ACCESS_PRIVATE</code>).
     * </p>
     *
     * @param modifier <code>int</code> constant for access level.
     * @see #ACCESS_PRIVATE
     * @see #ACCESS_PROTECTED
     * @see #ACCESS_PUBLIC
     * @see #STORAGE_STATIC
     * @see #MUTABILITY_VOLATILE
     * @see #MUTABILITY_FINAL
     * @see #MUTABILITY_SYNCHRONIZED                         
     */    
    public BitSet getModifier();
    
    /**
     * <p>
     *  This will return the Java <code>String</code> representation
     *    of this <code>Property</code>'s modifier. For
     *    example, <code>{@link #ACCESS_PRIVATE}</code> would be
     *    converted to "private".
     * </p>
     *
     * @param modifier <code>int</code> constant for access level.
     * @see #ACCESS_PRIVATE
     * @see #ACCESS_PROTECTED
     * @see #ACCESS_PUBLIC
     * @see #STORAGE_STATIC
     * @see #MUTABILITY_VOLATILE
     * @see #MUTABILITY_FINAL
     * @see #MUTABILITY_SYNCHRONIZED                         
     */
    public String getModifierString();
    
    /**
     * <p>
     *  This will whether or not this <code>Property</code>
     *    is a <code>Collection</code> (in other words, the property
     *    represents a collection of values). By default, properties
     *    are all singular values.
     * </p>
     *
     * @param isCollection <code>true</code> is multiple values can be stored,
     *        or else <code>false</code>.
     */
    public void setIsCollection(boolean isCollection);
    
    /**
     * <p>
     *  This will indicate whether this <code>Property</code> represents
     *    a <code>Collection</code> of values (resulting in a <code>true</code>
     *    result from this method), or a singular value (resulting in a
     *    <code>false</code> result).
     * </p>
     *
     * @return <code>boolean</code> - whether or not this <code>Property</code>
     *         represents a <code>Collection<code>.
     */
    public boolean isCollection();

    /**
     * <p>
     *  This indicates whether or not this <code>Property</code> has a
     *    a default value set.
     * </p>
     *
     * @return <code>boolean</code> - whether there is a default value set.
     */ 
    public boolean hasDefaultValue();
    
    /**
     * <p>
     *  This will set the default value of the property. Since no typing
     *    is available at this point, a simple Java <code>Object</code>
     *    is allowed as the type supplied. As a result, any errors in 
     *    mismatches between object type and allowed paramater type will
     *    occur at runtime, when class generation takes place. Supplying
     *    a value here essentially results in:
     * <code><pre>
     *    public class Foo {
     *
     *        private String myString = "some default value";
     *
     *        public String getMyString() {
     *            return myString;
     *        }
     *
     *        public void setMyString(String myString) {
     *            this.myString = myString;
     *        }
     * 
     *        // Other methods and properties
     *    }
     * </pre></code>
     * </p><p>
     *  Also, note that data binding users who supply their own class
     *    implementations will <b>LOSE THIS DEFAULT VALUE</b>, as the interface
     *    alone cannot specify a default value. So use this carefully!
     * </p>
     *
     * @param defaultValue <code>Object</code> to be used as default value.
     */
    public void setDefaultValue(Object defaultValue);
    
    /**
     * <p>
     *  This will retrieve the default value associated with this property,
     *    or <code>null</code> if there is not one. For more information on
     *    default property values, see <code>{@link #setDefaultValue(Object)}</code>.
     * </p>
     *
     * @return <code>Object</code> - default value of the property.
     */
    public Object getDefaultValue();
    
    /**
     * <p>
     *  This will indicate if this <code>Property</code> has a set of
     *    allowed values (an enumeration) specified for it.
     * </p>
     *
     * @return <code>boolean</code> - whether an enumeration is specified.
     */
    public boolean hasEnumeration();

    /**
     * <p>
     *  This will set a list (enumeration) of allowed values for this
     *    <code>Property</code>.
     * </p>
     *
     * @param enumeration the <code>Vector</code> of allowed values.
     */
    public void setEnumeration(Vector enumeration);
   
    /**
     * <p>
     *  This returns the <code>Vector</code> of allowed values for this
     *    <code>Property</code>, or <code>null</code> if there is none.
     * </p>
     *
     * @return <code>Vector</code> - the allowed values.
     */
    public Vector getEnumeration();
        
}
