/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.generator;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;
import java.util.BitSet;

// Zeus imports
import org.enhydra.zeus.Binding;
import org.enhydra.zeus.Generator;
import org.enhydra.zeus.ZeusDefaults;
import org.enhydra.zeus.binding.AtomicProperty;
import org.enhydra.zeus.binding.Container;
import org.enhydra.zeus.binding.Property;
import org.enhydra.zeus.util.ClassUtils;
import org.enhydra.zeus.util.NamingUtils;

/**
 * <p>
 *  <code>{@link Generator}</code> is the portion of Zeus that will convert
 *    a set of constraints, visible through a <code>{@link Binding}</code>,
 *    and output them as generated Java classes (to a 
 *    <code>File</code> [directory]).
 * </p><p>
 *  <code>SimpleGenerator</code> is the simplest form of a 
 *    <code>Generator</code> and takes in either a single <code>Binding</code>
 *    or a <code>List</code> of them, and directly generates Java classes.
 * </p>
 *
 * @author Brett McLaughlin
 * @author Maciej Zawadzki
 * @version 1.0
 */
public class SimpleGenerator implements Generator {
    
    /** The <code>Collection</code> class to use */
    protected String collectionClass;

    /** The package name of the interface/implementation classes */
    protected String javaPackage;

    /** This determines if the class should be serializable */
    protected boolean generateAsSerializable;

    /** A list of bindings to ignore */
    protected List ignoredBindings;

    /** The package name of the interface/implementation classes */
    protected String interfacePackage;

    /** The package name for the implementation classes. */
    protected String implPackage;
    
    /** Directory where all generated interfaces should be placed. */
    protected File interfaceDir;
    
    /**
     * Directory where all generated implementation 
     *   classes should be placed.
     */
    protected File implDir;
    
    /** Base directory for generated files. */
    protected File outputDir;
    
    /** Keeps track of whether the Generator needs to be reinitialized. */
    protected boolean initialized;

    /**
     * <p>
     *  This will construct a new instance of <code>SimpleGenerator</code>.
     * </p>
     */
    public SimpleGenerator() {
        interfacePackage = "";
        implPackage = "";
        interfaceDir = null;
        implDir = null;
        outputDir = null;
        initialized = false;
        collectionClass = Generator.DEFAULT_COLLECTION_CLASS;
        generateAsSerializable = false;
        ignoredBindings = new LinkedList();
    }
    
    /**
     * <p>
     *  This will set the <code>Collection</code> class to use anytime that
     *    a collection of values need to be stored. The default value for
     *    this class is stored in <code>{@link #DEFAULT_COLLECTION_CLASS}</code>.
     *    The class specified should be an implementation of the
     *    <code>java.util.Collection</code> interface.
     * </p><p>
     *  Note that the supplied class name <i>must</i> be fully qualified. For
     *    example, <code>java.util.List</code> would be fine, but
     *   <code>List</code> would not be.
     * </p>
     *
     * @param collectionClass <code>String</code> name of collection to use.
     */
    public void setCollectionClass(String collectionClass) {
        if (collectionClass != null) {
            this.collectionClass = collectionClass;
            uninitialize();
        }
    }
    
    /**
     * <p>
     *  This will return the current class used for <code>Collection</code>
     *    properties.
     * </p>
     *
     * @return <code>String</code> - name of class used for <code>Collection</code>s.
     */
    public String getCollectionClass() { 
        return collectionClass;
    }

    /**
     * <p>
     *  Sets the Java package to generate new interfaces and implementation
     *    within. For example, supplying the value <tt>com.foo.bar</tt> 
     *    to this method and then generating classes, the classes 
     *    would all be in the <tt>com.foo.bar</tt> package after generation.
     * </p>
     * 
     * @param javaPackage <code>String</code> the Java package to 
     *        generate interfaces and implementation classes within.
     */
    public void setJavaPackage(String javaPackage) {
        setInterfacePackage(javaPackage);
        setImplementationPackage(javaPackage);
    }

    /**
     * <p>
     *  Returns the Java package to generate interfaces within.
     * </p>
     * 
     * @return <code>String</code> - the Java package to unmarshall 
     *         interfaces to.
     */
    public String getInterfacePackage() {
        return interfacePackage;
    }

    /**
     * <p>
     *  Sets the Java package to generate new interfaces within. For example,
     *    supplying the value <tt>com.foo.bar</tt> to this method and then
     *    generating classes, the interfaces would all be in the 
     *    <tt>com.foo.bar</tt> package after generation.
     * </p>
     * 
     * @param interfacePackage <code>String</code> the Java package to 
     *        generate interfaces within.
     */
    public void setInterfacePackage(String interfacePackage) {
        if (interfacePackage != null) {
            this.interfacePackage = interfacePackage;
            uninitialize();
        }
    }

    /**
     * <p>
     *  Returns the Java package to generate implementation classes
     *    within.
     * </p>
     * 
     * @return <code>String</code> - the Java package to generate 
     *         implementation classes within.
     */
    public String getImplementationPackage() {
        return implPackage;
    }

    /**
     * <p>
     *  Sets the Java package to generate new interfaces within. For example,
     *    supplying the value <tt>com.foo.bar</tt> to this method and then
     *    generating classes, the interfafces would all be in the 
     *    <tt>com.foo.bar</tt> package after generation.
     * </p>
     * 
     * @param implPackage <code>String</code> the Java package to 
     *        generate interfaces within.
     */
    public void setImplementationPackage(String implPackage) {
        if (implPackage != null) {
            this.implPackage = implPackage;
            uninitialize();
        }
    }

    /**
     * <p>
     *  This sets the output directory to output generated classes to.
     * </p>
     *
     * @param baseDir the name of the base directory where 
     *        all generated classes should be placed
     */
    public void setOutputDirectory(String outputDir) {
        if (outputDir != null) {
            this.outputDir = new File(outputDir);
            uninitialize();
        }
    }
    
    /**
     * <p>
     *  This sets the output directory to output generated classes to.
     * </p>
     *
     * @param outputDir the <code>File</code> (directory) where 
     *        all generated classes should be placed
     */
    public void setOutputDirectory(File outputDir) {
        if (outputDir != null) {
            this.outputDir = outputDir;
            uninitialize();
        }
    }

    /**
     * <p>
     *  This will return <code>true</code> if the generator is set to
     *    generate <code>Serializable</code> classes, or else
     *    <code>false</code>.
     * </p>
     *
     * @return <code>boolean</code> - whether classes will be serializable.
     */
    public boolean generateAsSerializable() {
        return generateAsSerializable;
    }

    /**
     * <p>
     *  This will allow for setting the generator to generate serializable
     *    classes. By default, classes are <i>not</i> set to be 
     *    serializable.
     * </p>
     *
     * @param generateAsSerializable whether or not to generate serializable
     *        classes
     */
    public void setGenerateAsSerializable(boolean generateAsSerializable) {
        this.generateAsSerializable = generateAsSerializable;
    }

    /**
     * <p>
     *  This allows a binding to be ignored. This is an intermediary step towards
     *    allowing binding schemas. The name of the binding should be passed
     *    in here, and no property generation for it will occur.
     * </p>
     *
     * @param bindingName the name of the binding to ignore.
     */
    public void setIgnoreBinding(String bindingName) {
        ignoredBindings.add(bindingName);
    }

    /**
     * <p>
     *  This will convert from a set of constraints to Java classes.
     *    It accesses those constraints through the
     *    <code>{@link Binding}</code> interface, which provides
     *    them in a representation-independent format, and then
     *    converts them into Java code.
     * </p>
     *
     * @param binder <code>Binder</code> with ability to convert from
     *        constraints to Zeus <code>Binding</code>s.
     * @throws <code>IOException</code> - when errors in writing to the
     *         supplied <code>Result</code> occur.
     */
    public void generate(Binding binding) throws IOException { 
        if (!isInitialized()) {
            initialize();
        }
    
        /**
         * Currently, only Containers can be generated (which
         *   includes ContainerProperties) as AtomicProperties
         *   won't result in classes.
         */
        if (binding instanceof Container) {
            Container container = (Container)binding;
        
            /** Generate interface */
            generateInterface(container);
        
            /** Generate implementation class */
            generateImplementation(container);            
        } else  {
            throw new IOException("Only containers can be used for class generation.");
        }
    }
    
    /**
     * <p>
     *  This will generate the interface (or interfaces, through recursion)
     *    for the supplied <code>Container</code>.
     * </p>
     *
     * @param container <code>Container</code> to generate classes from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
    private void generateInterface(Container container) 
        throws IOException {
        
        // Get a new File handle
        String filename =
            new StringBuffer(container.getJavaName())
                .append(".java")
                .toString();
        File outputFile = new File(interfaceDir, filename);
        FileWriter writer = new FileWriter(outputFile);

        if (getInterfacePackage().equals("") == false) {
            writer.write("package");
            writer.write(" ");
            writer.write(getInterfacePackage());
            writer.write(";\n\n");
        }

        // Make serializable if needed
        if (generateAsSerializable) {
            writer.write("import java.io.Serializable;\n\n");
        }
        
        writer.write("public interface ");
        writer.write(container.getJavaName());

        // Make serializable if needed
        if (generateAsSerializable) {
            writer.write(" extends Serializable");
        }

        writer.write(" {\n\n");

        // Write out element and attribute ordering
        generateElementAndAttributeArrays(container, writer);
        
        // Write out each method
        List properties = container.getProperties();        
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            if ((ignoredBindings.contains(property.getXMLName())) ||
                (property.getXMLName().startsWith("xmlns"))) {
                continue;
            }
            generatePropertyMethodDecl(property, writer);
        }        
        
        writer.write("}\n");    
        writer.flush();
        writer.close();
    }
    
    /**
     * <p>
     *  This will generate the implemenation (or implemenations, through recursion)
     *    for the supplied <code>Container</code>. 
     * </p>
     *
     * @param container <code>Container</code> to generate classes from.
     * @throws <code>IOException</code> - when generation errors occur.
     */
    private void generateImplementation(Container container) 
        throws IOException {
        
        // Get a new File handle
        String filename =
            new StringBuffer(container.getJavaName())
                .append("Impl.java")
                .toString();
        File outputFile = new File(implDir, filename);
        FileWriter writer = new FileWriter(outputFile);

        if (getImplementationPackage().equals("") == false) {
            writer.write("package");
            writer.write(" ");
            writer.write(getImplementationPackage());
            writer.write(";\n\n");
        }

        if (!getInterfacePackage().equals(getImplementationPackage())) {
            writer.write("import");
            writer.write(" ");
            writer.write(getInterfacePackage());
            writer.write(".*;\n\n");
        }
                
        writer.write("public class ");
        writer.write(container.getJavaName());
        writer.write("Impl implements ");
        writer.write(container.getJavaName());
        writer.write(" {\n\n");
        
        // Write out each property
        List properties = container.getProperties();
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            if ((ignoredBindings.contains(property.getXMLName())) ||
                (property.getXMLName().startsWith("xmlns"))) {
                continue;
            }
            generateProperty(property, writer);
        }
        
        writer.write("\n");
        
        writer.write("    public ");
        writer.write(container.getJavaName());
        writer.write("Impl() {\n");
        // Initialize any List variables needed
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();   
            if ((ignoredBindings.contains(property.getXMLName())) ||
                (property.getXMLName().startsWith("xmlns"))) {
                continue;
            }

            if (property.isCollection()) {
                writer.write("        ");
                writer.write(NamingUtils.getJavaCollectionVariableName(
                    property.getJavaName()));
                writer.write("List = new ");
                writer.write(ClassUtils.getCollectionImplClass(
                    collectionClass));
                writer.write("();\n");
            }
        }
        writer.write("    }\n\n");        
        
        // Write out each method implementation
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            if ((ignoredBindings.contains(property.getXMLName())) ||
                (property.getXMLName().startsWith("xmlns"))) {
                continue;
            }
            generatePropertyMethodImpl(property, writer);
        }        
        
        writer.write("}\n");    
        writer.flush();
        writer.close();
    }    
    
    /**
     * <p>
     *  This will take a <code>Property</code> and generate the code
     *    for that property as a member variable.
     * </p>
     *
     * @param property <code>Property</code> to generate code for.
     * @param writer <code>Writer</code> to write variable declaration to.
     * @throws <code>IOException</code> - when errors in writing to the
     *         supplied <code>Result</code> occur.
     */    
    private void generateProperty(Property property, Writer writer) 
        throws IOException {
            
        // See if this is a Collection or not
        String javaType = NamingUtils.getJavaClassName(property.getJavaType());
        boolean isCollection = property.isCollection();                        
        
        /**
         * There is a conditional here, although all that ever should
         *   get to here are AtomicProperty objects. But just in case...
         */
        if (property instanceof AtomicProperty) {            
            AtomicProperty atomicProperty = (AtomicProperty)property;
            // Write out the property
            writer.write("    ");
            writer.write(atomicProperty.getModifierString());
            writer.write(" ");
            if (isCollection) {
                writer.write(collectionClass);
            } else {
                writer.write(NamingUtils.getJavaClassName(
                    atomicProperty.getJavaType()));
            }
            writer.write(" ");

            if (isCollection) {
                writer.write(NamingUtils.getJavaCollectionVariableName(
                    atomicProperty.getJavaName()));
                writer.write("List");
            } else {
                writer.write(NamingUtils.getJavaVariableName(
                    atomicProperty.getJavaName()));
            }
            
            // if the property has a default value, initialize the variable
            
            if (atomicProperty.hasDefaultValue()) {
               writer.write(" = \"");
               writer.write((String)atomicProperty.getDefaultValue());
               writer.write("\"");
            }
            writer.write(";\n");
            
            // if the property has an enumeration, generate variables to
            //   hold the data to validate incoming data on set operations
            
            if (atomicProperty.hasEnumeration())
            {
               String myName = atomicProperty.getJavaName();
               writer.write("    private static boolean valid");
               writer.write(myName);
               writer.write("Initialized = false;\n");
               writer.write("    private static java.util.Vector valid");
               writer.write(myName);
               writer.write(" = null;\n");
            }  
        }
    }

    /**
     * <p>
     *  This will take a <code>Property</code> and generate the code
     *    for that property as an accessor/mutator method (definition only).
     * </p>
     *
     * @param property <code>Property</code> to generate methods for.
     * @param writer <code>Writer</code> to write method declarations to.
     * @throws <code>IOException</code> - when errors in writing to the
     *         supplied <code>Result</code> occur.
     */       
    private void generatePropertyMethodDecl(Property property, Writer writer) 
        throws IOException {
        
        // See if this is a Collection or not
        String javaType = NamingUtils.getJavaClassName(
            property.getJavaType());
        boolean isCollection = property.isCollection();        
        
        /**
         * There is a conditional here, although all that ever should
         *   get to here are AtomicProperty objects. But just in case...
         */
        if (property instanceof AtomicProperty) {
            AtomicProperty atomicProperty = (AtomicProperty)property;
            
            // Write out the property accessor
            writer.write("    public ");
            if (isCollection) {
                writer.write(collectionClass);
            } else {
                writer.write(javaType);
            }
            writer.write(" get");
            writer.write(atomicProperty.getJavaName());
            if (isCollection) {
                writer.write("List");
            }            
            writer.write("();\n\n");
            
            // Write out the property mutator if the property is not #FIXED
            //   #FIXED attributes cannot be changed...
            if (!atomicProperty.getModifier().get(Property.MUTABILITY_FINAL))
            {
               writer.write("    public void set");
               writer.write(atomicProperty.getJavaName());
               if (isCollection) {
                   writer.write("List");
               }            
               writer.write("(");
               if (isCollection) {
                   writer.write(collectionClass);
               } else {
                   writer.write(javaType);
               }            
               writer.write(" ");
               if (isCollection) {
                   writer.write(NamingUtils.getJavaCollectionVariableName(
                       atomicProperty.getJavaName()));
                   writer.write("List");
               } else {
                   writer.write(NamingUtils.getJavaVariableName(
                       atomicProperty.getJavaName()));
               }
               
               /**
                * If the property has an enumerator, we want to throw 
                *   an IllegalArgumentException if the incoming data on 
                *   set operations doesn't pass the validation
                */               
               if (atomicProperty.hasEnumeration()) {
                   writer.write(")\n       throws java.lang.IllegalArgumentException;\n");
               } else {
                   writer.write(");\n\n");
               }
               
               // If this is a Collection, add the "add" method
               if (isCollection) {
                   writer.write("    public void add");
                   writer.write(atomicProperty.getJavaName());
                   writer.write("(");
                   writer.write(javaType);
                   writer.write(" ");
                   writer.write(NamingUtils.getJavaVariableName(
                       atomicProperty.getJavaName()));
                   writer.write(");\n\n");

                   // add the "remove" method                   
                   writer.write("    public void remove");
                   writer.write(atomicProperty.getJavaName());
                   writer.write("(");
                   writer.write(javaType);
                   writer.write(" ");
                   writer.write(NamingUtils.getJavaVariableName(
                       atomicProperty.getJavaName()));
                   writer.write(");\n\n");
                   
               }
               
           }               
        }
    }    
    
    /**
     * <p>
     *  This will take a <code>Property</code> and generate the code
     *    for that property as an accessor/mutator method (with implementation).
     * </p>
     *
     * @param property <code>Property</code> to generate code for.
     * @param writer <code>Writer</code> to write method implementations to.
     * @throws <code>IOException</code> - when errors in writing to the
     *         supplied <code>Result</code> occur.
     */       
    private void generatePropertyMethodImpl(Property property, Writer writer) 
        throws IOException {
        
        // See if this is a Collection or not
        String javaType = NamingUtils.getJavaClassName(property.getJavaType());
        boolean isCollection = property.isCollection();
        
        /**
         * There is a conditional here, although all that ever should
         *   get to here are AtomicProperty objects. But just in case...
         */
        if (property instanceof AtomicProperty) {
            AtomicProperty atomicProperty = (AtomicProperty)property;
            
            // Write out the property accessor
            writer.write("    public ");
            if (isCollection) {
                writer.write(collectionClass);
            } else {
                writer.write(javaType);
            }            
            writer.write(" get");
            writer.write(atomicProperty.getJavaName());
            if (isCollection) {
                writer.write("List");
            }
            writer.write("() {\n");
            writer.write("        return ");

            if (isCollection) {
                writer.write(NamingUtils.getJavaCollectionVariableName(
                    atomicProperty.getJavaName()));
                writer.write("List");
            } else {
                writer.write(NamingUtils.getJavaVariableName(
                    atomicProperty.getJavaName()));
            }
            writer.write(";\n");    
            writer.write("    }\n\n");
            
            // Write out the property mutator if the property is not #FIXED            
            if (!atomicProperty.getModifier().get(Property.MUTABILITY_FINAL))
            {
                writer.write("    public void set");
                writer.write(atomicProperty.getJavaName());
                if (isCollection) {
                    writer.write("List");
                }            
                writer.write("(");
                if (isCollection) {
                    writer.write(collectionClass);
                } else {
                    writer.write(javaType);
                }            
                writer.write(" ");
                if (isCollection) {
                    writer.write(NamingUtils.getJavaCollectionVariableName(
                        atomicProperty.getJavaName()));
                    writer.write("List");
                } else {
                    writer.write(NamingUtils.getJavaVariableName(
                        atomicProperty.getJavaName()));
                }

                /**
                 * If the property has an enumeration, then generate 
                 *   the code to initialize the validation data and to 
                 *   validate incoming data
                 */
                if (atomicProperty.hasEnumeration()) {
                   String myName = atomicProperty.getJavaName();
                   String flagName = 
                       new StringBuffer("valid").append(myName)
                                                .append("Initialized")
                                                .toString();
                   String vectorName = 
                       new StringBuffer("valid").append(myName)
                                                .toString();
                   Vector theEnumeration = atomicProperty.getEnumeration();

                   writer.write(")\n        ");
                   writer.write("throws java.lang.IllegalArgumentException {\n");
                   writer.write("        if (!");
                   writer.write(flagName);
                   writer.write(") {\n");
                   writer.write("            ");
                   writer.write(vectorName);
                   writer.write(" = new java.util.Vector();\n");
                   for (int i=0, size=theEnumeration.size(); i < size; i++) {
                       writer.write("            ");
                       writer.write(vectorName);
                       writer.write(".addElement(\"");
                       writer.write((String)theEnumeration.elementAt(i));
                       writer.write("\");\n");
                   }
                   writer.write("            ");
                   writer.write(flagName);
                   writer.write(" = true;\n");
                   writer.write("        }\n");
                   writer.write("        if (!");
                   writer.write(vectorName);
                   writer.write(".contains(");
                   writer.write(NamingUtils.getJavaVariableName(myName));
                   writer.write("))\n");
                   writer.write("              throw new ");
                   writer.write("java.lang.IllegalArgumentException(\"");
                   writer.write("Illegal value for attribute ");
                   writer.write(myName);
                   writer.write("\");\n");
                   writer.write("        else\n");
                   writer.write("              this.");
                   writer.write(NamingUtils.getJavaVariableName(myName));
                   writer.write(" = ");
                   writer.write(NamingUtils.getJavaVariableName(myName));
                   writer.write(";\n");                       
                } else {    
                   writer.write(") {\n");                
                   writer.write("        this.");

                   if (isCollection) {
                       writer.write(NamingUtils.getJavaCollectionVariableName(
                           atomicProperty.getJavaName()));
                       writer.write("List");
                   } else {
                       writer.write(NamingUtils.getJavaVariableName(
                           atomicProperty.getJavaName()));
                   }

                   writer.write(" = ");

                   if (isCollection) {
                       writer.write(NamingUtils.getJavaCollectionVariableName(
                           atomicProperty.getJavaName()));
                       writer.write("List");
                   } else {
                       writer.write(NamingUtils.getJavaVariableName(
                           atomicProperty.getJavaName()));
                   }

                   writer.write(";\n");
                }                   
                writer.write("    }\n\n");            
            
                // If this is a Collection, add the "add" method
                if (isCollection) {
                    writer.write("    public void add");
                    writer.write(atomicProperty.getJavaName());
                    writer.write("(");
                    writer.write(javaType);
                    writer.write(" ");
                    writer.write(NamingUtils.getJavaVariableName(
                        atomicProperty.getJavaName()));
                    writer.write(") {\n");
                    writer.write("        ");
                    writer.write(NamingUtils.getJavaCollectionVariableName(
                        atomicProperty.getJavaName()));
                    writer.write("List.add(");
                    writer.write(NamingUtils.getJavaVariableName(
                        atomicProperty.getJavaName()));
                    writer.write(");\n");
                    writer.write("    }\n\n");
                    
                    // add the "remove" method

                    writer.write("    public void remove");
                    writer.write(atomicProperty.getJavaName());
                    writer.write("(");
                    writer.write(javaType);
                    writer.write(" ");
                    writer.write(NamingUtils.getJavaVariableName(
                        atomicProperty.getJavaName()));
                    writer.write(") {\n");
                    writer.write("        ");
                    writer.write(NamingUtils.getJavaCollectionVariableName(
                        atomicProperty.getJavaName()));
                    writer.write("List.remove(");
                    writer.write(NamingUtils.getJavaVariableName(
                        atomicProperty.getJavaName()));
                    writer.write(");\n");
                    writer.write("    }\n\n");
                }
            }     
        }
    }    
    
    /**
     * <p>
     *  This generates an array of element and attribute names, for
     *    correct ordering in marshalling.
     * </p>
     *
     * @param container <code>Container</code> with properties 
     *        to generate
     * @param writer means of output
     * @throws <code>IOException</code> when output errors occur.
     */
    private void generateElementAndAttributeArrays(
        Container container, Writer writer) throws IOException {

        // Initialize variables
        List properties = container.getProperties();
        StringBuffer attributeNames = new StringBuffer();
        boolean firstAtt = true;
        StringBuffer elementNames = new StringBuffer();
        boolean firstElem = true;        
        BitSet t = null;
        
        // construct the contents of the arrays 
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            t = property.getModifier();

            // See if it's an attribute            
            if (t.get(Property.SOURCE_ATTLIST)) {
                // append the comma separator
                if (firstAtt) {
                    firstAtt = false;
                } else {
                    attributeNames.append(", ");
                }
                    
                // append the name of the attribute
                attributeNames.append("\"");
                attributeNames.append(property.getXMLName());
                attributeNames.append("\"");

            // See if it's an element
            } else if (t.get(Property.SOURCE_ELEMENT)) {
                // append the comma separator
                if (firstElem) {
                    firstElem = false;
                } else {
                    elementNames.append(", ");
                }
                    
                // append the name of the element
                elementNames.append("\"");
                elementNames.append(property.getXMLName());
                elementNames.append("\"");
            }
        }
        
        writer.write("    public static final String ");
        writer.write(ZeusDefaults.XML_NAME_FIELD);
        writer.write(" = \"");
        writer.write(container.getXMLName());
        writer.write("\";\n");

        writer.write("    public static final String[] ");
        writer.write(ZeusDefaults.ATTRIBUTE_ARRAY_FIELD);
        writer.write(" = {");
        writer.write(attributeNames.toString());
        writer.write("};\n");
        
        writer.write("    public static final String[] ");
        writer.write(ZeusDefaults.ELEMENT_ARRAY_FIELD);
        writer.write(" = {");
        writer.write(elementNames.toString());
        writer.write("};\n\n");
        
    }

    /**
     * <p>
     *  This will initialize this instance of <code>Generator</code>,
     *    handling tasks like directory creation.
     * </p>
     *
     * @throws <code>IOException</code> - when problems related to
     *         setting up files and directories arise.
     */    
    private void initialize() throws IOException {        
        // deal with the null situation
        if (outputDir == null) {
            outputDir = new File(".");
        }
        
        /** Ensure that we have a directory */
        if ((outputDir.exists()) && (!outputDir.isDirectory())) {
            throw new IOException(
                "The specified output direcotry was not a valid directory");
        }

        // create the directory if it does not exist
        if (!outputDir.exists()) {
            outputDir.mkdir();
        }

        String dir = null;
        String rem = null;
        interfaceDir = outputDir;
        if ((interfacePackage != null) && (!outputDir.equals(""))) {
            // create directory for interfaces
            rem = interfacePackage;
            dir = null;
            do {
                if (rem.indexOf(".") > 0) {
                    dir = rem.substring(0, rem.indexOf("."));
                    rem = rem.substring(rem.indexOf(".") + 1);
                } else {
                    dir = rem;
                }
                interfaceDir = new File(interfaceDir, dir);
                if (!interfaceDir.exists()) {
                    interfaceDir.mkdir();
                }
            } while (!dir.equals(rem)) ;
        }

        implDir = outputDir;
        if ((implPackage != null) && (!implPackage.equals(""))) {
            // create the directory for implementation classes
            rem = implPackage;
            dir = null;
            do {
                if (rem.indexOf(".") > 0) {
                    dir = rem.substring(0, rem.indexOf("."));
                    rem = rem.substring(rem.indexOf(".") + 1);
                } else {
                    dir = rem;
                }
                implDir = new File(implDir, dir);
                if (!implDir.exists()) {
                    implDir.mkdir();
                }
            } while (!dir.equals(rem)) ;

        } else {
            /**
             * The user did not specify a different impl directory, 
             *   so use the interface settings
             */
            implDir = interfaceDir;
            implPackage = interfacePackage;
        }

        initialized = true;
    }        

    /**
     * <p>
     *  This will reset the initialization state of this <code>Generator</code>
     *    instance to <code>false</code>.
     * </p>
     */    
    private void uninitialize() {
        initialized = false;
    }
    
    /**
     * <p>
     *  This will indicate if this <code>Generator</code> instance has been
     *    initialized.
     * </p>
     *
     * @return <code>boolean</code> - whether this instance is initialized.
     */
    private boolean isInitialized() {
        return initialized;
    }
}

