/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.result;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;

// Zeus imports
import org.enhydra.zeus.Result;

/**
 * <p>
 *  <code>{@link Result}</code> provides an interface for all output
 *    means. It details the required contract that other
 *    portions of the Zeus XML data binding framework must
 *    use for processing to an arbitrary output.
 * </p>
 * <p>
 *  This implementation of <code>Result</code> deals with
 *    output to Java I/O streams, such as <code>OutputStream</code>s
 *    and <code>Writer</code>s. It allows passing in of these 
 *    types of constructs and subsequent output to them, through
 *    various constructors.
 * </p>
 *
 * @author Brett McLaughlin
 * @version 1.0
 */
public class StreamResult extends BaseResult {

    /** <code>Writer</code> to write to */
    private Writer writer;

    /**
     * <p>
     *  This will take in an <code>OutputStream</code>
     *    and output XML to that stream when asked. It
     *    also takes in a system ID for resolution of
     *    external references.
     * </p>
     *
     * @param outputStream <code>OutputStream</code> to write to.
     * @param systemID <code>String</code> system ID for output 
     *        document.
     */
    public StreamResult(OutputStream outputStream, String systemID) {
        this.writer = new OutputStreamWriter(outputStream);
        setSystemID(systemID);
    }

    /**
     * <p>
     *  This will take in an <code>OutputStream</code>
     *    and output XML to that stream when asked
     *    It assumes that no system ID is present; for
     *    passing in a system ID, see
     *    <code>{@link #StreamResult(OutputStream, String)}</code>.
     * </p>
     *
     * @param outputStream <code>OutputStream</code> to write to.
     */
    public StreamResult(OutputStream outputStream) {
        this(outputStream, null);
    }
    
    /**
     * <p>
     *  This will take in a <code>Reader</code>
     *    and write XML to that stream when asked. It
     *    also takes in a system ID for resolution of
     *    external references.
     * </p>
     *
     * @param writer <code>Writer</code> to write to.
     * @param systemID <code>String</code> system ID for input 
     *        document.
     */
    public StreamResult(Writer writer, String systemID) {
        this.writer = writer;
        setSystemID(systemID);
    }

    /**
     * <p>
     *  This will take in a <code>Writer</code>
     *    and write XML to that stream when asked.
     *    It assumes that no system ID is present; for
     *    passing in a system ID, see
     *    <code>{@link #StreamResult(Writer, String)}</code>.
     * </p>
     *
     * @param reader <code>Writer</code> to write to.
     */
    public StreamResult(Writer writer) {
        this(writer, null);
    }

    /**
     * <p>
     *  This will write a character stream to the output
     *    facility associated with this <code>Result</code>.
     *    No additional formatting is added, so line feeds,
     *    tabs, or other special characters should be handled
     *    by the code performing output.
     * </p>
     *
     * @param output <code>String</code> to output.
     * @throws <code>IOException</code> - when errors in output occur.
     */
    public void write(String output) throws IOException {
        if (writer != null) {
            writer.write(output);
            writer.flush();
        } else {
            throw new IOException("No output source to write to!");
        }
    }
    
    /**
     * <p>
     *  This will expose the <code>Writer</code> associated with this
     *    <code>StreamResult</code> for direct manipulation.
     * </p>
     *
     * @return <code>Writer</code> - the output method for this <code>Result</code>.
     */
    public Writer getWriter() throws IOException {
        if (writer != null) {
            return writer;
        } else {
            throw new IOException("No output source supplied.");
        }
    }
}