/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.source;

import java.io.IOException;

// Zeus imports
import org.enhydra.zeus.Source;

// JDOM imports
import org.jdom.Document;

// SAX imports
import org.xml.sax.EntityResolver;

/**
 * <p>
 *  <code>{@link Source}</code> provides an interface for all input
 *    means. It details the required contract that other
 *    portions of the Zeus XML data binding framework must
 *    use for processing of an arbitrary input.
 * </p>
 * <p>
 *  This implementation of <code>Source</code> handles the
 *    basic functionality of dealing with system IDs so that
 *    other implementations don't have to code these methods.
 *    Thus, <code>XXXSource</code> classes should extend
 *    this class, rather than directly implementing
 *    <code>Source</code>, and will get this functionality
 *    "for free."
 * </p>
 *
 * @author Brett McLaughlin
 * @version 1.0
 */
public abstract class BaseSource implements Source {

    /** The system ID for this <code>Source</code> */
    protected String systemID;

    /** <code>EntityResolver</code> to pass to parser */
    protected EntityResolver entityResolver;
        
    /**
     * <p>
     *  This will return the system ID associated with
     *    this <code>Source</code>. This is generally in
     *    the form of a URI.
     * </p>
     *
     * @return <code>String</code> - the system ID for the
     *         <code>Source</code>.
     */
    public String getSystemID() {
        return systemID;
    }
    
    /**
     * <p>
     *  This will set the system ID for this 
     *    <code>Source</code>. This is important to use,
     *    even when input is in the form of an input stream
     *    (see <code>{@link StreamSource}</code>)
     *    for resolving external references, such as to a DTD.
     * </p>
     *
     * @param systemID <code>String</code> system ID to use.
     */
    public void setSystemID(String systemID) {
        this.systemID = systemID;
    }    
    
    /**
     * <p>
     *  This will return a JDOM <code>Document</code> that
     *    represents the input source.
     * </p>
     *
     * @return <code>Document</code> - the JDOM representation
     *         of the input source.
     * @throws <code>IOException</code> - when construction of
     *         a <code>Document</code> generates errors.
     */
    public abstract Document getDocument() throws IOException;    

    /**
     * <p>
     *  This will set an EntityResolver for this 
     *    <code>Source</code>. This is passed to the underlying
     *    parser to use.
     * </p>
     *
     * @param entityResolver <code>org.xml.sax.EntityResolver</code> Entity 
     * Resolver to use.
     */    
     public void setEntityResolver(EntityResolver entityResolver) { 
        this.entityResolver = entityResolver;
     }
}