/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.source;

import java.io.IOException;

// Zeus imports
import org.enhydra.zeus.Source;

// JDOM imports
import org.jdom.Document;

// DTDParser imports
import com.wutka.dtd.DTD;

/**
 * <p>
 *  <code>{@link Source}</code> provides an interface for all input
 *    means. It details the required contract that other
 *    portions of the Zeus XML data binding framework must
 *    use for processing of an arbitrary input.
 * </p>
 * <p>
 *  <code>DTDSource</code> extends the default <code>Source</code> implementation
 *    in <code>{@link BaseSource}</code>, but provides functionality
 *    specific to obtaining DTD information. Since all other XML constraint
 *    methodologies involve constraints that are themselves well-formed
 *    XML, handling DTDs is a special case, and therefore needs a special
 *    class (this one!).
 * </p><p>
 *  It should be noted that this class will <b>always</b> return
 *    <code>null</code> on the method <code>{@link #getDocument()}</code>.
 *    Instead, it adds a method, <code>{@link #getDTD()}</code>, for
 *    returning a Java representation of an XML DTD.
 * </p>
 *
 * @author Brett McLaughlin
 * @version 1.0
 */
public abstract class DTDSource extends BaseSource { 
    
    /**
     * <p>
     *  This will <b>always</b> return <code>null</code>
     *    since there is no way to represent an XML DTD 
     *    as a JDOM <code>Document</code> (or any XML
     *    document representation).
     * </p>
     *
     * @return <code>Document</code> - the JDOM representation
     *         of the input source.
     * @throws <code>IOException</code> - when construction of
     *         a <code>Document</code> generates errors.
     */
    public Document getDocument() throws IOException {
        return null;
    }
    
    /**
     * <p>
     *  This will return the <code>DTDParser</code>'s 
     *    <code>DTD</code> representation of the supplied
     *    XML DTD.
     * </p>
     *
     * @return <code>DTD</code> - the <code>DTDParser</code>
     *         representation of the input source.
     * @throws <code>IOException</code> - when construction of
     *         a <code>DTD</code> generates errors.
     */
    public abstract DTD getDTD() throws IOException;

}