/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.source;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.Reader;

// Zeus imports
import org.enhydra.zeus.Source;

// JDOM imports
import org.jdom.Document;

// DTDParser imports
import com.wutka.dtd.DTD;
import com.wutka.dtd.DTDParser;

/**
 * <p>
 *  <code>{@link Source}</code> provides an interface for all input
 *    means. It details the required contract that other
 *    portions of the Zeus XML data binding framework must
 *    use for processing of an arbitrary input.
 * </p>
 * <p>
 *  This implementation of <code>DTDSource</code> deals with
 *    input from Java I/O streams, such as <code>InputStream</code>s
 *    and <code>Reader</code>s. It allows passing in of these 
 *    types of constructs and subsequent reading of them, through
 *    various constructors.
 * </p>
 *
 * @author Brett McLaughlin
 * @version 1.0
 */
public class StreamDTDSource extends DTDSource {
    
    /** <code>Reader</code> to read from */
    private Reader reader;

    /** <code>DTD</code> constructed from input */
    private DTD dtd;

    /**
     * <p>
     *  This will take in an <code>InputStream</code>
     *    and read XML from that stream when asked,
     *    converting the read XML into a
     *    <code>{@link Document}</code> as needed. It
     *    also takes in a system ID for resolution of
     *    external references.
     * </p>
     *
     * @param inputStream <code>InputStream</code> to read from.
     * @param systemID <code>String</code> system ID for input 
     *        document.
     */
    public StreamDTDSource(InputStream inputStream, String systemID) {
        this.reader = new InputStreamReader(inputStream);
        setSystemID(systemID);
    }

    /**
     * <p>
     *  This will take in an <code>InputStream</code>
     *    and read XML from that stream when asked,
     *    converting the read XML into a
     *    <code>{@link Document}</code> as needed.
     *    It assumes that no system ID is present; for
     *    passing in a system ID, see
     *    <code>{@link #StreamDTDSource(InputStream, String)}</code>.
     * </p>
     *
     * @param inputStream <code>InputStream</code> to read from.
     */
    public StreamDTDSource(InputStream inputStream) {
        this(inputStream, null);
    }
    
    /**
     * <p>
     *  This will take in a <code>Reader</code>
     *    and read XML from that stream when asked,
     *    converting the read XML into a
     *    <code>{@link Document}</code> as needed. It
     *    also takes in a system ID for resolution of
     *    external references.
     * </p>
     *
     * @param reader <code>Reader</code> to read from.
     * @param systemID <code>String</code> system ID for input 
     *        document.
     */
    public StreamDTDSource(Reader reader, String systemID) {
        this.reader = reader;
        setSystemID(systemID);
    }

    /**
     * <p>
     *  This will take in a <code>Reader</code>
     *    and read XML from that stream when asked,
     *    converting the read XML into a
     *    <code>{@link Document}</code> as needed.
     *    It assumes that no system ID is present; for
     *    passing in a system ID, see
     *    <code>{@link #StreamDTDSource(Reader, String)}</code>.
     * </p>
     *
     * @param reader <code>Reader</code> to read from.
     */
    public StreamDTDSource(Reader reader) {
        this(reader, null);
    }
    
    /**
     * <p>
     *  This will return the <code>DTDParser</code>'s 
     *    <code>DTD</code> representation of the supplied
     *    XML DTD.
     * </p>
     *
     * @return <code>DTD</code> - the <code>DTDParser</code>
     *         representation of the input source.
     * @throws <code>IOException</code> - when construction of
     *         a <code>DTD</code> generates errors.
     */
    public DTD getDTD() throws IOException {
        DTDParser dtdParser = new DTDParser(reader);
        return dtdParser.parse();
    }
}