/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.source;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.Reader;

// Zeus imports
import org.enhydra.zeus.Source;

// JDOM imports
import org.jdom.Document;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

/**
 * <p>
 *  <code>{@link Source}</code> provides an interface for all input
 *    means. It details the required contract that other
 *    portions of the Zeus XML data binding framework must
 *    use for processing of an arbitrary input.
 * </p>
 * <p>
 *  This implementation of <code>Source</code> deals with
 *    input from Java I/O streams, such as <code>InputStream</code>s
 *    and <code>Reader</code>s. It allows passing in of these 
 *    types of constructs and subsequent reading of them, through
 *    various constructors.
 * </p>
 *
 * @author Brett McLaughlin
 * @version 1.0
 */
public class StreamSource extends BaseSource {
    
    /** <code>Reader</code> to read from */
    private Reader reader;

    /** <code>Document</code> constructed from input */
    private Document document;

    /**
     * <p>
     *  This will take in an <code>InputStream</code>
     *    and read XML from that stream when asked,
     *    converting the read XML into a
     *    <code>{@link Document}</code> as needed. It
     *    also takes in a system ID for resolution of
     *    external references.
     * </p>
     *
     * @param inputStream <code>InputStream</code> to read from.
     * @param systemID <code>String</code> system ID for input 
     *        document.
     */
    public StreamSource(InputStream inputStream, String systemID) {
        this.reader = new InputStreamReader(inputStream);
        setSystemID(systemID);
    }

    /**
     * <p>
     *  This will take in an <code>InputStream</code>
     *    and read XML from that stream when asked,
     *    converting the read XML into a
     *    <code>{@link Document}</code> as needed.
     *    It assumes that no system ID is present; for
     *    passing in a system ID, see
     *    <code>{@link #StreamSource(InputStream, String)}</code>.
     * </p>
     *
     * @param inputStream <code>InputStream</code> to read from.
     */
    public StreamSource(InputStream inputStream) {
        this(inputStream, null);
    }
    
    /**
     * <p>
     *  This will take in a <code>Reader</code>
     *    and read XML from that stream when asked,
     *    converting the read XML into a
     *    <code>{@link Document}</code> as needed. It
     *    also takes in a system ID for resolution of
     *    external references.
     * </p>
     *
     * @param reader <code>Reader</code> to read from.
     * @param systemID <code>String</code> system ID for input 
     *        document.
     */
    public StreamSource(Reader reader, String systemID) {
        this.reader = reader;
        setSystemID(systemID);
    }

    /**
     * <p>
     *  This will take in a <code>Reader</code>
     *    and read XML from that stream when asked,
     *    converting the read XML into a
     *    <code>{@link Document}</code> as needed.
     *    It assumes that no system ID is present; for
     *    passing in a system ID, see
     *    <code>{@link #StreamSource(Reader, String)}</code>.
     * </p>
     *
     * @param reader <code>Reader</code> to read from.
     */
    public StreamSource(Reader reader) {
        this(reader, null);
    }
    
    /**
     * <p>
     *  This will return a JDOM <code>Document</code> that
     *    represents the input source.
     * </p>
     *
     * @return <code>Document</code> - the JDOM representation
     *         of the input source.
     * @throws <code>IOException</code> - when construction of
     *         a <code>Document</code> generates errors.     
     */
    public Document getDocument() throws IOException {
        try {
            SAXBuilder builder = new SAXBuilder();
            if (entityResolver != null) {
                builder.setEntityResolver(entityResolver);
            }
            if (reader != null) {
                document = builder.build(reader);
            } else {
                throw new IOException("No source to read XML from!");
            }
    
            return document;
        } catch (JDOMException e) {
            throw new IOException(e.getMessage());
        }
    }
}
