/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.Reader;
import java.util.Iterator;
import java.util.List;

// Zeus imports
import org.enhydra.zeus.Binding;
import org.enhydra.zeus.Generator;
import org.enhydra.zeus.generator.SimpleGenerator;

/** 
 * <p>
 *  This provides basic functionality for classes that need to
 *    implement the <code>{@link SourceGenerator}</code> interface.
 * </p>
 *
 * @author Brett McLaughlin
 */
public abstract class BaseSourceGenerator {

    /** The reader for reading constraints from */
    protected Reader constraintsReader;

    /** The directory to output classes to */
    protected File outputDir;

    /** The package to generate classes within */
    protected String javaPackage;

    /** Whether or not to generate classes as serializable */
    protected boolean generateAsSerializable;

    /**
     * Prefix to be prepended to the name of every generated 
     *   interface and class.
     */
    protected String namePrefix = "";
    
    /**
     * Determines whether simple elements map to data members 
     *   on parent clases or to their own classes.
     */
    protected boolean collapseSimpleElements = false;
    /** The package name of the interface/implementation classes */
    protected String interfacePackage;

    /**
     * <p>
     *  This sets up defaults for the generator.
     * </p>
     */
    public BaseSourceGenerator() {
        javaPackage = "";
        generateAsSerializable = false;
        outputDir = new File(".");
    }

    /**
     * <p>
     *  This allows for supplying the constraints file to use
     *    for source code generation as a <code>String</code> URI.
     * </p>
     *
     * @param fileURI <code>String</code> URI for constraints file.
     * @throws <code>IOException</code> - when the specified file URI
     *         is invalid.
     */
    public void setConstraintsInput(String fileURI) throws IOException {
        FileReader reader = new FileReader(
            new File(fileURI));
        setConstraintsInput(reader);
    }

    /**
     * <p>
     *  This allows for supplying the constraints file to use
     *    for source code generation as a <code>File</code>.
     * </p>
     *
     * @param file <code>File</code> to read constraints from.
     * @throws <code>IOException</code> - when the specified file URI
     *         is invalid.
     */
    public void setConstraintsInput(File file) throws IOException {
        setConstraintsInput(new FileReader(file));
    }

    /**
     * <p>
     *  This allows for supplying the constraints file to use
     *    for source code generation as a <code>FileReader</code>.
     * </p>
     *
     * @param reader <code>Reader</code> to read constraints
     *               from.
     */
    public void setConstraintsInput(Reader reader) {
        constraintsReader = reader;
    }

    /**
     * <p>
     *  This allows for supplying the constraints file to use
     *    for source code generation as an 
     *    <code>InputStream</code>.
     * </p>
     *
     * @param inputStream <code>InputStream</code> to read
     *        constraints from.
     */
    public void setConstraintsInput(InputStream inputStream) {
        setConstraintsInput(new InputStreamReader(inputStream));
    }

    /**
     * <p>
     *  This allows specification of an output directory for the
     *    generated classes.
     * </p>
     *
     * @param outputDirString <code>String</code> specifying output 
     *        directory for generated classes.
     * @throws <code>IOException</code> - when invalid direcotory is
     *         specified
     */
    public void setOutputDir(String outputDir) throws IOException {
        setOutputDir(new File(outputDir));
    }

    /**
     * <p>
     *  This allows specification of an output directory for the
     *    generated classes.
     * </p>
     *
     * @param outputDir <code>File</code> specifying output directory
     *        for generated classes.
     * @throws <code>IOException</code> - when invalid direcotory is
     *         specified
     */
    public void setOutputDir(File outputDir) throws IOException {
        if ((!outputDir.exists()) || (!outputDir.isDirectory())) {
            throw new IOException("Output directory must exist before " +
                "class generation.");
        }
        this.outputDir = outputDir;
    }

    /**
     * <p>
     *  This will set the package for generating classes within.
     * </p>
     *
     * @param javaPackage the package to generate classes within.
     */
    public void setJavaPackage(String javaPackage) {
        this.javaPackage = javaPackage;
    }

    /**
     * <p>
     *  This will set whether or not to set source code classes as
     *    serializable (implementing <code>java.io.Serializable</code>.
     * </p>
     *
     * @param generateAsSerializable <code>boolean</code> indicating 
     *        whether to generate the source code as serializable.
     */
    public void setGenerateAsSerializable(boolean generateAsSerializable) {
        this.generateAsSerializable = generateAsSerializable;
    }

    /**
     * <p>
     *  Sets the Java package to generate new interfaces within. For example,
     *    supplying the value <tt>com.foo.bar</tt> to this method and then
     *    generating classes, the interfaces would all be in the 
     *    <tt>com.foo.bar</tt> package after generation.
     * </p>
     * 
     * @param interfacePackage <code>String</code> the Java package to 
     *        generate interfaces within.
     */
    public void setInterfacePackage(String interfacePackage) {
        this.interfacePackage = interfacePackage;
    }
    /**
     *
     * <p>
     *
     * This method allows the developer to set a prefix that
     *
     *   will be prepended to the name of every generated
     *
     *   interface and class.
     *
     * </p>
     *
     *
     *
     * @param String prefix that will be prepended to the
     *
     *       name of every generated interface and class
     *
     */
    public void setNamePrefix(String namePrefix) {
        this.namePrefix = namePrefix;
    }
    
    /**
     *
     * <p>
     *
     * This method allows the developer to specify that all
     *
     *   simple XML elements (elements with
     *
     *   <code>#PCDATA</code> content) should map to data
     *
     *   members on parent member classes rather than their
     *
     *   own classes.
     *
     * </p>
     *
     *
     *
     * @param collapseSimpleElements true if simple elements
     *
     *       should be collapsed
     *
     */
    public void setCollapseSimpleElements(boolean collapseSimpleElements) {
        this.collapseSimpleElements = collapseSimpleElements;
    }
    
    /**
     * <p>
     *  This method performs class generation.
     * </p>
     *
     * @throws <code>IOException</code> - when class generation fails
     */
    public void generate() throws IOException {        
        // Generator for source code
        Generator generator = new SimpleGenerator();
        generator.setJavaPackage(javaPackage);
        generator.setInterfacePackage(interfacePackage);
        generator.setOutputDirectory(outputDir);
        generator.setGenerateAsSerializable(generateAsSerializable);

        List bindings = getConstraintBindings();
        for (Iterator i = bindings.iterator(); i.hasNext(); ) {
            Binding binding = (Binding)i.next();
            generator.generate(binding);
        }
    }

    /**
     * <p>
     *  This defines a contract for subclasses to allow them to get the
     *    Zeus <code>{@link Binding}</code> objects to generate code
     *    from.
     * </p>
     *
     * @return <code>List</code> - a list of the <code>{@link Binding}</code>
     *         objects to generate code from.
     * @throws <code>IOException</code> - when binding creation fails
     */
    protected abstract List getConstraintBindings() throws IOException;
}