/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.util;

import java.io.IOException;
import java.util.List;

// Zeus imports
import org.enhydra.zeus.Binder;
import org.enhydra.zeus.binder.DTDBinder;
import org.enhydra.zeus.source.DTDSource;
import org.enhydra.zeus.source.StreamDTDSource;

/** 
 * <p>
 *  This is a standalone "utility" class. It allows source code
 *    generation from a DTD.
 * </p>
 *
 * @author Brett McLaughlin
 */
public class DTDSourceGenerator extends BaseSourceGenerator
    implements SourceGenerator {

    /**
     * <p>
     *  This sets up defaults for the generator.
     * </p>
     */
    public DTDSourceGenerator() {
        super();
    }

    /**
     * <p>
     *  This defines a contract for subclasses to allow them to get the
     *    Zeus <code>{@link org.enhydra.zeus.Binding}</code> objects to generate code
     *    from.
     * </p>
     *
     * @return <code>List</code> - a list of the Binding
     *         objects to generate code from.
     * @throws <code>IOException</code> - when binding creation fails
     */
    protected List getConstraintBindings() throws IOException {
        DTDSource dtdSource = new StreamDTDSource(constraintsReader);
        Binder dtdBinder = new DTDBinder(dtdSource);
        dtdBinder.setNamePrefix(namePrefix);
        dtdBinder.setCollapseSimpleElements(collapseSimpleElements);
       
        return dtdBinder.getBindings();
    }

    /**
     * <p>
     *  This provides command-line class generation.
     * </p>
     *
     * @param args <code>String</code> array of command-line arguments.
     */
    public static void main(String[] args) {
        try {
            Arguments arguments = new Arguments(args);

            // Generate source
            SourceGenerator sourceGenerator =
                new DTDSourceGenerator();

            // Ensure correct usage
            String constraintsFilename = 
                arguments.getValue("constraints");
            if (constraintsFilename == null) {
                System.out.println("Usage: java " +
                    sourceGenerator.getClass().getName() +
                    " -constraints=<constraints filename>" +
                    " [-outputDir=<output directory>" +
                    " [-package=<Java package name>]" +
                     " [-interfacePackage=<package for generated interfaces>]" +
                     " [-serializable=<true | false>]" +
                     " [-namePrefix=<generated class name prefix>]" +
                     " [-collapseSimple=<true | false>]");
                return;
            }

            // Get command-line options
            String outputDirName = arguments.getValue("outputDir");
            String javaPackage = arguments.getValue("package");
            String serializable = arguments.getValue("serializable");
            String interfacePackage = arguments.getValue("interfacePackage");
            String namePrefix = arguments.getValue("namePrefix");
            String collapseSimple = arguments.getValue("collapseSimple");
            sourceGenerator.setConstraintsInput(constraintsFilename);
            if (outputDirName != null) {
                sourceGenerator.setOutputDir(outputDirName);
            }
            if (javaPackage != null) {
                sourceGenerator.setJavaPackage(javaPackage);
            }
            if ((serializable != null) && 
                (serializable.equalsIgnoreCase("true"))) {
                sourceGenerator.setGenerateAsSerializable(true);
            } else {
                sourceGenerator.setGenerateAsSerializable(false);
            }
            if (interfacePackage != null) {
                sourceGenerator.setInterfacePackage(interfacePackage);
            }
            if (namePrefix != null) {
                sourceGenerator.setNamePrefix(namePrefix);
            }
            if (collapseSimple != null && 
                collapseSimple.equalsIgnoreCase("true")) {
                sourceGenerator.setCollapseSimpleElements(true);
            }
            sourceGenerator.generate();

        } catch (IOException e) {
            System.out.println("Error generating source code: " +
                e.getMessage());
            e.printStackTrace();
        }
    }
}
