/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.util;

import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * <code>NamingUtils</code> is a Zeus utility class that provides methods
 * which deal with the naming issues involved in data binding.  Specifically,
 * this class will deal with deriving Java class names, deriving Java variable
 * names, deriving XML element names, and validating legal Java identifiers.
 * <p>
 *
 * @author Robert Sese
 * @author Maciej Zawadzki
 * @version 1.0
 */
public class NamingUtils {

    /** Prefix to append to names that clash with Java reserved words. */
    public static final String XML_PREFIX = "XML";

    /** The default alternative representation for illegal characters. */
    public static final String DEFAULT_MAPPING = "";

    /**
     * A map of illegal characters mapped to legal alternative representations.
     */
    private static Map illegalCharMap = new HashMap();

    /** A map of all the Java reserved words. */
    private static HashMap reservedWords = new HashMap();

    /** An array of all the Java reserved words to be stored in a map. */ 
    private static String reservedWordsArray[] = { "abstract",
        "boolean", "break", "byte", "case", "catch", "char", "class",
        "char", "const", "continue", "default", "do", "double", "else",
        "extends", "false", "final", "finally", "float", "for",
        "goto", "if", "implements", "import", "instanceof", "int",
        "interface", "long", "native", "new", "null", "package",
        "private", "protected", "public", "return", "short", "static",
        "super", "switch", "synchronized", "this", "throw", "throws",
        "transient", "true", "try", "void", "volatile", "while" };

    // Initialize the reserved words map and the illegal characters map.
    static {
        for (int i = 0; i < reservedWordsArray.length; i++) {
            reservedWords.put(reservedWordsArray[i], new Integer(i));
        }
        reservedWordsArray = null;

        // XXX: Add more illegal chars mappings...we don't have to check too
        // many because we can assume XML naming rules?
        illegalCharMap.put(new Character('.'), DEFAULT_MAPPING);
        illegalCharMap.put(new Character('-'), DEFAULT_MAPPING);
        illegalCharMap.put(new Character(':'), DEFAULT_MAPPING);
    }

    /**
     * <p>
     * Prevent construction.
     * </p>
     */
    private NamingUtils() {}

    /**
     * <p>
     * Returns the value that the given key is mapped to.  This value will
     * replace any occurence of the key when a name contains illegal characters
     * for a Java identifier.
     * </p>
     *
     * @param key the key mapped to the value to return.
     * @return <code>String</code> - the value the given key is mapped to.
     */
    public static String getCharMapping(Character key) {
        return (String) illegalCharMap.get(key);
    }

    /**
     * <p>
     * Maps the given key to the given value.  This value will replace any
     * occurence of the key when  a name contains illegal characters
     * for a Java identifier.
     * </p>
     *
     * @param key the key to map the given value to.
     * @param value the value to map the given key to.
     */
    public static void setCharMapping(Character key, String value) {
        illegalCharMap.put(key, value);
    }

    /**
     * <p>
     * Returns the conventional Java class name from an XML name.  The initial
     * character is capitalized, and any illegal characters are replaced.
     * There is no check for a collision with a Java reserved word because all
     * Java reserved words have their initial character lower case.  For
     * example, calling getJavaClassName("my-element") would return "MyElement"
     * as the Java class name.
     * </p>
     *
     * @param xmlName the XML name to convert to a Java class name.
     * @return <code>String</code> - the converted Java name.
     */
    public static String getJavaClassName(String xmlName) {
        return convertIllegalChars(CapitalizationUtils.initialUpper(xmlName));
    }

    /**
     * <p>
     * Returns the conventional Java variable name from a Java class name.  The
     * initial character is uncapitalized, and any illegal characters are
     * replaced.  If there is a collision with a Java reserved word, the name
     * is prefixed with <code>{@link #XML_PREFIX}</code>.  For example, calling
     * getJavaVariableName("MyClass") would return "myClass" as the Java
     * variable name.
     * </p>
     *
     * @param className the class name to convert to a variable name.
     * @return <code>String</code> - the converted Java variable name.
     */
    public static String getJavaVariableName(String className) {
        return convertIllegalChars(
            convertReservedWord(CapitalizationUtils.initialLower(className)));
    }

    /**
     * <p>
     * Returns the Java variable name from a Java class name assuming that the
     * variable will be a collection based variable (i.e. ending with "List" as
     * in "myVariableList").  The initial character is uncapitalized, and any
     * illegal characters are replaced.  There is no check for a collision with
     * a Java reserved word because the assumption is that the variable has
     * some word appended to it that identifies the fact that the variable
     * denotes a collection.  For example, calling
     * getJavaCollectionVariableName("MyClass") would return "myClass" as the
     * Java variable name.
     * </p>
     *
     * @param className the class name to convert to a collection based
     *        variable name.
     * @return <code>String</code> - the converted Java variable name.
     */
    public static String getJavaCollectionVariableName(String className) {
        return convertIllegalChars(CapitalizationUtils.initialLower(className));
    }

    /**
     * <p>
     * Returns the XML name from a typical accessor method.  The naming
     * convention for such a method is "get&lt;variable-name&gt;".  The first
     * three characters are discarded, and the initial character is
     * uncapitalized.  For example, calling
     * getXMLElementNameFromAccessor("getMyVariable") would return "myVariable"
     * as the XML name.  This method would also work with typical mutator
     * methods.
     * </p>
     *
     * @param accessor the name of the accessor method to derive the XML name
     *        from.
     * @return <code>String</code> - the converted XML name.
     */
    public static String getXMLElementNameFromAccessor(String accessor) {
        /**
         * XXX: REVISIT - MBZ - This will produce a problem.  
         *   The whole issue of naming should be revisited especially 
         *   now that we have the original names of the elements 
         *   and attributes available via the arrays on the interface.
         */
        return accessor.substring(3);
        //return CapitalizationUtils.initialLower(accessor.substring(3));
    }

    /**
     * <p>
     * Checks whether the given name collides with a Java reserved word.  If
     * there is a collision, the name is returned prefixed with
     * <code>{@link #XML_PREFIX}</code>, else it is returned unchanged.
     * </p> 
     *    
     * @param name the name to check.
     * @return <code>String</code> - the converted name if there is a collision
     *         with a Java reserved word, else the same name.
     */
    private static String convertReservedWord(String name) {
        if (reservedWords.get(name) == null) {
            return name;
        } else {
            return new StringBuffer().append(XML_PREFIX)
                .append(name).toString();
        }
    }

    /**
     * <p>
     * Checks whether each character in the given name is valid for a Java
     * identifier.  If there are illegal characters, the name is returned with
     * each illegal character replaced with its value in the internal map of
     * illegal <code>Character</code>s to replacement <code>String</code>s,
     * else the same name is returned.  If the illegal character can be
     * interpreted as a separator character (such as the '-' in my-attribute),
     * then the character following the separator character will be capitalized
     * (i.e. my-attribute becomes myAttribute).
     * </p> 
     *    
     * @param name the name to check.
     * @return <code>String</code> - the converted name if illegal characters 
     *         are present, else the same name.
     */
    private static String convertIllegalChars(String name) {
        StringBuffer verifiedName = new StringBuffer();
        char[] characters = name.toCharArray();

        // XXX: Uppercase the next letter after all illegal chars, or just the
        // ones that can be interpreted as separator chars?
        boolean upperNext = false;

        for (int i = 0; i < characters.length; i++) {
            char c = characters[i];

            if (i == 0) {
                if (Character.isJavaIdentifierStart(c) == false) {
                    String legalValue =
                        (String) illegalCharMap.get(new Character(c));

                    if (legalValue == null) {

                        // XXX: Throw an exception or append DEFAULT_MAPPING?
                        throw new IllegalArgumentException("No mapping for " +
                            "illegal character: " + c);
                    } else {
                        verifiedName.append(legalValue);

                        // XXX Does it make sense to uppercase the next char
                        // if the illegal char is the first char of the
                        // identifier?
                        upperNext = true;
                    }
                } else {
                    verifiedName.append(c);
                }
            } else {
                if (Character.isJavaIdentifierPart(c) == false) {
                    String legalValue =
                        (String) illegalCharMap.get(new Character(c));

                    if (legalValue == null) {

                        // XXX: Throw an exception or append DEFAULT_MAPPING?
                        throw new IllegalArgumentException("No mapping for " +
                            "illegal character: " + c);
                    } else {
                        verifiedName.append(legalValue);
                        upperNext = true;
                    }
                } else {
                    if (upperNext) {
                        verifiedName.append(Character.toUpperCase(c));
                        upperNext = false;
                    } else {
                        verifiedName.append(c);
                    }
                }
            }
        }
        return verifiedName.toString();
    }

}
