/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.util;

import java.io.File;
import java.io.InputStream;
import java.io.IOException;
import java.io.Reader;

/** 
 * <p>
 *  This is a standalone "utility" interface. It allows source code
 *    generation from a set of XML constraints. It also allows for
 *    providing a standard set of methods that all utility source
 *    code generation classes should provide (at a minimum).
 * </p>
 *
 * @author Brett McLaughlin
 */
public interface SourceGenerator {

    /**
     * <p>
     *  This allows for supplying the constraints file to use
     *    for source code generation as a <code>String</code> URI.
     * </p>
     *
     * @param fileURI <code>String</code> URI for constraints file.
     * @throws <code>IOException</code> - when the specified file URI
     *         is invalid.
     */
    public void setConstraintsInput(String fileURI) throws IOException;

    /**
     * <p>
     *  This allows for supplying the constraints file to use
     *    for source code generation as a <code>File</code>.
     * </p>
     *
     * @param file <code>File</code> to read constraints from.
     * @throws <code>IOException</code> - when the specified file URI
     *         is invalid.
     */
    public void setConstraintsInput(File file) throws IOException;

    /**
     * <p>
     *  This allows for supplying the constraints file to use
     *    for source code generation as a <code>FileReader</code>.
     * </p>
     *
     * @param reader <code>Reader</code> to read constraints
     *               from.
     */
    public void setConstraintsInput(Reader reader);

    /**
     * <p>
     *  This allows for supplying the constraints file to use
     *    for source code generation as an 
     *    <code>InputStream</code>.
     * </p>
     *
     * @param inputStream <code>InputStream</code> to read
     *        constraints from.
     */
    public void setConstraintsInput(InputStream inputStream);

    /**
     * <p>
     *  This allows specification of an output directory for the
     *    generated classes.
     * </p>
     *
     * @param outputDir <code>String</code> specifying output directory
     *        for generated classes.
     */
    public void setOutputDir(String outputDir) throws IOException;

    /**
     * <p>
     *  This allows specification of an output directory for the
     *    generated classes.
     * </p>
     *
     * @param outputDir <code>File</code> specifying output directory
     *        for generated classes.
     */
    public void setOutputDir(File outputDir) throws IOException;

    /**
     * <p>
     *  This will set the package for generating classes within.
     * </p>
     *
     * @param javaPackage the package to generate classes within.
     */
    public void setJavaPackage(String javaPackage);

    /**
     * <p>
     *  Sets the Java package to generate new interfaces within. For example,
     *    supplying the value <tt>com.foo.bar</tt> to this method and then
     *    generating classes, the interfaces would all be in the 
     *    <tt>com.foo.bar</tt> package after generation.
     * </p>
     * 
     * @param interfacePackage <code>String</code> the Java package to 
     *        generate interfaces within.
     */
    public void setInterfacePackage(String interfacePackage);
    /**
     * <p>
     *  This method allows the developer to set a prefix that 
     *    will be prepended to the name of every generated 
     *    interface and class.  
     * </p>
     *
     * @param String prefix that will be prepended to the 
     *        name of every generated interface and class
     */
    public void setNamePrefix(String namePrefix);
    /**
     * <p>
     *  This method allows the developer to specify that all 
     *    simple XML elements (elements with 
     *    <code>#PCDATA</code> content) should map to data 
     *    members on parent member classes rather than their 
     *    own classes.
     * </p>
     *
     * @param collapseSimpleElements true if simple elements 
     *        should be collapsed
     */
    public void setCollapseSimpleElements(
        boolean collapseSimpleElements);
    /**
     * <p>
     *  This will set whether or not to set source code classes as
     *    serializable (implementing <code>java.io.Serializable</code>.
     * </p>
     *
     * @param generateAsSerializable <code>boolean</code> indicating 
     *        whether to generate the source code as serializable.
     */
    public void setGenerateAsSerializable(boolean generateAsSerializable);

    /**
     * <p>
     *  This method performs class generation.
     * </p>
     *
     * @throws <code>IOException</code> - when class generation fails
     */
    public void generate() throws IOException;
}