/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.binding;

import java.util.Iterator;
import java.util.List;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.enhydra.zeus.AbstractBindingTest;
import org.enhydra.zeus.Binding;

/**
 * <p>
 * Abstract test case for Container objects.  This class provides tests for
 * methods on the Container interface that all Container subclasses will
 * inherit.
 * </p>
 *
 * <p>
 * Concrete Container test cases should extend this class, override the
 * createContainer method, and add any tests specific to the class being
 * tested.
 * </p>
 *
 * @author Robert Sese
 * @version 1.0
 */
public abstract class AbstractContainerTest extends AbstractBindingTest {

    /**
     * Creates a new Container object that will be used to test Container level
     * methods.
     *
     * @return a Container object to test with.
     */
    protected abstract Container createContainer();

    /**
     * Constructs a new AbstractContainerTest.
     *
     * @param name Name of the test case.
     */
    public AbstractContainerTest(String name) {
        super(name);
    }

    /**
     * Creates a new Binding object that will be used to test Binding level
     * methods.
     *
     * @return a Binding object to test with.
     */
    public Binding createBinding() {

        // Just delegate to the Container subclass.
        return createContainer();
    }

    /**
     * Tests adding a Property to a Container.
     */ 
    public void testAddProperty() {
        Container container = createContainer();
        Property property = new AtomicProperty("MyName", "MyType");
        container.addProperty(property);

        /*
         * XXX: Do we need to define equals for Bindings, or is comparsion by
         * name enough?
         */

        List properties = container.getProperties();
        boolean hasProp = false;
        for (Iterator iter = properties.iterator(); iter.hasNext(); ) {
            Property p = (Property) iter.next();
            if (p.getJavaName().equals(property.getJavaName())) {
                hasProp = true;
                break;
            }
        }
        assert(hasProp == true);
    }

    /**
     * Tests removing a Property from a Container.
     */ 
    public void testRemoveProperty() {
        Container container = createContainer();
        Property property = new AtomicProperty("MyName", "MyType");
        container.addProperty(property);
        container.removeProperty(property.getJavaName());

        /*
         * XXX: Do we need to define equals for Bindings, or is comparsion by
         * name enough?
         */

        List properties = container.getProperties();
        boolean hasProp = false;
        for (Iterator iter = properties.iterator(); iter.hasNext(); ) {
            Property p = (Property) iter.next();
            if (p.getJavaName().equals(property.getJavaName())) {
                hasProp = true;
                break;
            }
        }
        assert(hasProp == false);
    }

    /**
     * Tests getting the Properties from a Container.
     */ 
    public void testGetProperties() {
        Container container = createContainer();
        Property property1 = new AtomicProperty("MyName", "MyType1");
        Property property2 = new AtomicProperty("MyName2", "MyType2");
        container.addProperty(property1);
        container.addProperty(property2);

        List properties = container.getProperties();
        boolean hasProp1 = false;
        boolean hasProp2 = false;
        for (Iterator iter = properties.iterator(); iter.hasNext(); ) {
            Property p = (Property) iter.next();
            if (p.getJavaName().equals(property1.getJavaName())) {
                hasProp1 = true;
            } else if (p.getJavaName().equals(property2.getJavaName())) {
                hasProp2 = true;
            }
        }
        assert(hasProp1 && hasProp2);
    }

    /**
     * Tests clearing the Properties in a Container.
     */ 
    public void testClearProperties() {
        Container container = createContainer();
        Property property1 = new AtomicProperty("MyName", "MyType1");
        Property property2 = new AtomicProperty("MyName2", "MyType2");
        container.addProperty(property1);
        container.addProperty(property2);
        container.clearProperties();

        List properties = container.getProperties();
        boolean hasProp1 = false;
        boolean hasProp2 = false;
        for (Iterator iter = properties.iterator(); iter.hasNext(); ) {
            Property p = (Property) iter.next();
            if (p.getJavaName().equals(property1.getJavaName())) {
                hasProp1 = true;
            } else if (p.getJavaName().equals(property2.getJavaName())) {
                hasProp2 = true;
            }
        }
        assert((!hasProp1) && (!hasProp2));
        assert(properties.size() == 0);
    }

}

