/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.binding;

import java.util.BitSet;
import java.util.Vector;

import junit.framework.TestCase;

import org.enhydra.zeus.AbstractBindingTest;
import org.enhydra.zeus.Binding;

/**
 * <p>
 * Abstract test case for Property objects.  This class provides tests for
 * methods on the Property interface that all Property subclasses will inherit.
 * </p>
 *
 * <p>
 * Concrete Property test cases should extend this class, override the
 * createProperty method, and add any tests specific to the class being tested.
 * </p>
 *
 * @author Robert Sese
 * @version 1.0
 */
public abstract class AbstractPropertyTest extends AbstractBindingTest {

    /**
     * Creates a new Property object that will be used to test Property level
     * methods.
     *
     * @return a Property object to test with.
     */
    protected abstract Property createProperty();

    /**
     * Constructs a new AbstractPropertyTest.
     *
     * @param name Name of the test case.
     */
    public AbstractPropertyTest(String name) {
        super(name);
    }

    /**
     * Creates a new Binding object that will be used to test Binding level
     * methods.
     *
     * @return a Binding object to test with.
     */
    public Binding createBinding() {

        // Delegate to the Property being tested.
        return createProperty();
    }

    /**
     * Tests setting the modifiers on a Property.
     */
    public void testSetModifier() {
        Property property = createProperty();
        BitSet modifiers = new BitSet();
        modifiers.set(Property.ACCESS_PRIVATE);
        modifiers.set(Property.SOURCE_ATTLIST);

        property.setModifier(modifiers);
        assertEquals(modifiers, property.getModifier());

        modifiers = new BitSet();
        modifiers.set(Property.ACCESS_PUBLIC);
        modifiers.set(Property.SOURCE_ELEMENT);

        property.setModifier(modifiers);
        assertEquals(modifiers, property.getModifier());
    }

    /**
     * Tests getting the modifiers from a Property.
     */
    public void testGetModifier() {
        Property property = createProperty();
        BitSet modifiers = new BitSet();
        modifiers.set(Property.ACCESS_PRIVATE);
        modifiers.set(Property.SOURCE_ATTLIST);

        property.setModifier(modifiers);
        assertEquals(modifiers, property.getModifier());
    }

    /**
     * Tests getting the modifiers from a Property as a String.
     */
    public void testGetModifierString() {
        Property property = createProperty();
        BitSet modifiers = new BitSet();
        modifiers.set(Property.ACCESS_PRIVATE);
        modifiers.set(Property.SOURCE_ATTLIST);
        String expected = "private";

        property.setModifier(modifiers);
        assertEquals(expected, property.getModifierString());
    }

    /**
     * Tests setting the isCollection property on a Property.
     */
    public void testSetIsCollection() {
        Property property = createProperty();
        boolean expected = true;
        property.setIsCollection(expected);
        assertEquals(expected, property.isCollection());

        expected = false;
        property.setIsCollection(expected);
        assertEquals(expected, property.isCollection());
    }

    /**
     * Tests getting the isCollection property from a Property.
     */
    public void testIsCollection() {
        Property property = createProperty();
        boolean expected = true;
        property.setIsCollection(expected);
        assertEquals(expected, property.isCollection());
    }

    /**
     * Tests getting the modifiers on a Property.
     */
    public void testHasDefaultValue() {
        Property property = createProperty();
        boolean expected = true;
        property.setDefaultValue("MyDefaultValue");
        assertEquals(expected, property.hasDefaultValue());
    }

    /**
     * Tests setting the default value on a Property.
     */
    public void testSetDefaultValue() {
        Property property = createProperty();
        String expected1 = "MyDefaultValue";
        property.setDefaultValue(expected1);
        assertEquals(expected1, (String) property.getDefaultValue());

        Integer expected2 = new Integer(10);
        property.setDefaultValue(expected2);
        assertEquals(expected2, (Integer) property.getDefaultValue());
    }

    /**
     * Tests getting the default value from a Property.
     */
    public void testGetDefaultValue() {
        Property property = createProperty();
        String expected1 = "MyDefaultValue";
        property.setDefaultValue(expected1);
        assertEquals(expected1, (String) property.getDefaultValue());
    }

    /**
     * Tests getting the hasEnumeration property from a Property.
     */
    public void testHasEnumeration() {
        Property property = createProperty();
        boolean expected = true;
        property.setEnumeration(new Vector());
        assertEquals(expected, property.hasEnumeration());
    }

    /**
     * Tests setting the hasEnumeration property on a Property.
     */
    public void testSetEnumeration() {
        Property property = createProperty();
        Vector expected1 = new Vector();
        expected1.add("MyValue1");
        expected1.add("MyValue2");
        property.setEnumeration(expected1);
        assertEquals(expected1, property.getEnumeration());

        Vector expected2 = new Vector();
        expected2.add(new Integer(1));
        expected2.add(new Integer(2));
        property.setEnumeration(expected2);
        assertEquals(expected2, property.getEnumeration());
    }

    /**
     * Tests getting the enumerated values from a Property.
     */
    public void testGetEnumeration() {
        Property property = createProperty();
        Vector expected1 = new Vector();
        expected1.add("MyValue1");
        expected1.add("MyValue2");
        property.setEnumeration(expected1);
        assertEquals(expected1, property.getEnumeration());
    }

}

