/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.source;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.Reader;
import java.io.StringBufferInputStream;
import java.io.StringReader;

// Zeus imports
import org.enhydra.zeus.Source;

// JDOM imports
import org.jdom.Document;

// JUnit imports
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * <p>
 *  <code>StreamSourceTest</code> provides rudimentary tests for 
 *    <code>{@link StreamSource}</code> class. 
 * </p>
 *
 * @author <a href="mailto:smarks@digisolutions.com">Spencer A Marks</a>
 * @version $Id: StreamSourceTest.java,v 1.1 2001/01/29 15:58:43 brett Exp $
 * @see TestCase
 */
public class StreamSourceTest extends TestCase {

    /**
     * <p>
     *  Creates a new <code>StreamResultTest</code> instance.
     *    Required by JUnit. <CODE>testName</CODE>
     *    is simply the name of the test.
     * </p>
     *
     * @param testName a <code>String</code> value
     */
    public StreamSourceTest(String testName) {
        super(testName);
    }

    /**
     * <p>
     *  This method maeks it easier to call these
     *    tests dynamically.
     * </p>
     *
     * @return <code>TestSuite</code> - corresponds to this 
     *         <code>TestCase</code>.
     */
    public static Test suite() {
        return new TestSuite(StreamSourceTest.class);
    }

    /**
     * <p>
     *  Test the various <CODE>{@link StreamSource}</CODE> 
     *    constructors.
     * </p>
     */
    public void testConstructors() {
        String systemID = "file:///test/system/ID.xml";
        StringBufferInputStream in = 
            new StringBufferInputStream("Test Input Stream");
        StreamSource ss = new StreamSource(in, systemID);
        ss = new StreamSource(in);
        StringReader reader = new StringReader("Test Input Reader");
        ss = new StreamSource(reader, systemID);
        ss = new StreamSource(reader);
    }
    
    /**
     * <p>
     *  Rudimentary test of the 
     *    <CODE>{@link StreamSource#getDocument()}</CODE> method.
     *    This is a negative test. Getting the document should
     *    fail and this should be detected and handled correctly. 
     * </p>
     */
    public void testGetDocumentNegative() {
        boolean caught = false;
        try {
            String systemID = "file:///test/system/ID.xml";
            StringBufferInputStream in =
                new StringBufferInputStream("Test Input Stream");
            StreamSource ss = new StreamSource(in, systemID);
            Document doc = ss.getDocument();
        } catch (IOException e) {
            caught = true;
        } finally {
            assert(caught);
        }
    }

    /**
     * <p>
     *  Rudimentary positive test of the 
     *    <CODE>{@link StreamSource#getDocument()}</CODE> method.
     *    A well-formed document should be returned.
     * </p>
     */
    public void testGetDocumentPositive() {
        // I need a robust way to get a xml document here
        // What is the plan for system properties type configuration?
    }


    /**
     * <p>
     *  Several small tests that exercise 
     *    <CODE>{@link StreamSource#getSystemID()}</CODE>.
     * </p>
     */
    public void testGetSystemID() {
        String systemID = "file:///test/system/ID.xml";
        StringBufferInputStream in = 
            new StringBufferInputStream("Test Input Stream");
        StreamSource ss = new StreamSource(in, systemID);
        assertEquals(systemID, ss.getSystemID());
        
        ss = new StreamSource(in);
        assertNull(ss.getSystemID());

        StringReader reader = new StringReader("Test Input Reader");
        ss = new StreamSource(reader, systemID);
        assertEquals(systemID, ss.getSystemID());

        ss = new StreamSource(reader);
        assertNull(ss.getSystemID());

    }
    
    /**
     * <p>
     *  Several small tests that exercise 
     *    <CODE>{@link StreamSource#setSystemID()}</CODE>.
     * </p>
     */
    public void testSetSystemID() {
        String systemID = "file:///test/system/ID.xml";
        StringBufferInputStream in = 
            new StringBufferInputStream("Test Input Stream");
        StreamSource ss = new StreamSource(in);
        assertNull(ss.getSystemID());
        ss.setSystemID(systemID);
        assertEquals(systemID, ss.getSystemID());

        ss = new StreamSource(in, systemID);
        assertEquals(systemID, ss.getSystemID());

        StringReader reader = new StringReader("Test Input Reader");
        ss = new StreamSource(reader);
        assertNull(ss.getSystemID());
        ss.setSystemID(systemID);
        assertEquals(systemID, ss.getSystemID());
    }

    /**
     * <p>
     *  This main method allows the running of this 
     *    test standalone. 
     * </p>
     *
     * @param args a <code>String[]</code> value
     */
    public static void main(String[] args) {
        junit.textui.TestRunner.run(suite());
    }    

}


