/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.util;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * <p>
 * Tests that the <code>{@link NamingUtils}</code> class appropriately handles
 * deriving Java class names, deriving Java variable names, deriving XML
 * element names, and validating legal Java identifiers.
 * </p>
 * 
 * @author Robert Sese
 * @version 1.0
 */
public class NamingUtilsTest extends TestCase {

    /** The '.' character. */
    private Character dotChar;
   
    /**
     * <p>
     * Constructs a new NamingUtilsTest with the given name.
     * </p>
     *
     * @param name the name of the test.
     */
    public NamingUtilsTest(String name) {
        super(name);
    }

    /**
     * <p>
     * Establishes the test fixture.
     * </p>
     */
    protected void setUp() {
        this.dotChar = new Character('.');
    }

    /**
     * <p>
     * Returns a <code>TestSuite</code> consisting of
     * <code>NamingUtilsTest</code>.
     * </p>
     *
     * @returns a <code>TestSuite</code> consisting of
     *         <code>NamingUtilsTest</code>.
     */
    public static Test suite(){
        return new TestSuite(NamingUtilsTest.class);
    }

    /**
     * <p>
     * Tests whether <code>{@link NamingUtils#getJavaClassName(String)}</code>
     * capitalizes the initial character, replaces an illegal character with
     * the default replacement string it is mapped to, and then capitalizes the
     * following character for a given name.
     * </p>
     */
    public void testGetJavaClassName1() {
        String name = "my-element";
        String expected = "MyElement";
        String actual = NamingUtils.getJavaClassName(name);

        assertEquals("actual: " + actual + ", expected: " +
            expected, actual, expected);
    }

    /**
     * <p>
     * Tests whether
     * <code>{@link NamingUtils#getJavaVariableName(String)}</code>
     * uncapitalizes the initial character, replaces an illegal character with
     * the default replacement string it is mapped to, and capitalizes the
     * following character for a given name.
     * </p>
     */
    public void testGetJavaVariableName1() {
        String name = "My-class";
        String expected = "myClass";
        String actual = NamingUtils.getJavaVariableName(name);

        assertEquals("actual: " + actual + ", expected: " +
            expected, actual, expected);
    }

    /**
     * <p>
     * Tests whether
     * <code>{@link NamingUtils#getJavaVariableName(String)}</code>
     * uncapitalizes the initial character and converts an identifier that
     * collides with a Java reserved word by prefixing it with
     * <code>{@link NamingUtils#XML_PREFIX}</code>.
     * </p>
     */
    public void testGetJavaVariableName2() {
        String name = "Import";
        String expected = "XMLimport";
        String actual = NamingUtils.getJavaVariableName(name);

        assertEquals("actual: " + actual + ", expected: " +
            expected, actual, expected);
    }

    /**
     * <p>
     * Tests whether
     * <code>{@link NamingUtils#getJavaCollectionVariableName(String)}</code>
     * uncapitalizes the initial character, replaces an illegal character with
     * the default replacement string it is mapped to, and capitalizes the
     * following character for a given name.
     * </p>
     */
    public void testGetJavaCollectionVariableName1() {
        String name = "My-class";
        String expected = "myClass";
        String actual = NamingUtils.getJavaCollectionVariableName(name);

        assertEquals("actual: " + actual + ", expected: " +
            expected, actual, expected);
    }

    /**
     * <p>
     * Tests whether
     * <code>{@link NamingUtils#getXMLElementNameFromAccessor(String)}</code>
     * extracts a name from a typical accessor method name and uncapitalizes
     * the initial character.
     * </p>
     */
    public void testGetXMLElementNameFromAccessor1() {
        String name = "getMyElement";
        String expected = "myElement";
        String actual = NamingUtils.getXMLElementNameFromAccessor(name);

        assertEquals("actual: " + actual + ", expected: " +
            expected, actual, expected);
    }

    /**
     * <p>
     * Static entry point that allows this test to be run from the command line.
     * </p>
     */
    public static void main(String[] args) {
        junit.textui.TestRunner.run(suite());
    }    

}
